@@ -39,8 +39,7 @@ class DimensionType(type):
 
     def __new__(mcs, name, bases, cls_dict):
         """Return a new class with predefined comparison operators"""
-        for method_name in ('__lt__', '__le__', '__gt__', '__ge__',
-                            '__eq__', '__ne__'):
+        for method_name in ('__lt__', '__le__', '__gt__', '__ge__'):
             if method_name not in cls_dict:
                 cls_dict[method_name] = mcs._make_operator(method_name)
         return type.__new__(mcs, name, bases, cls_dict)
@@ -53,11 +52,7 @@ class DimensionType(type):
         def operator(self, other):
             """Operator delegating to the :class:`float` method `method_name`"""
             float_operator = getattr(float, method_name)
-            try:
-                float_other = float(other)
-            except (ValueError, TypeError):
-                return False
-            return float_operator(float(self), float_other)
+            return float_operator(float(self), float(other))
         return operator
 
 
@@ -70,7 +65,7 @@ class DimensionBase(AcceptNoneAttributeType, metaclass=DimensionType):
     :class:`float`.
 
     The internal representation is in terms of PostScript points. A PostScript
-    point is equal to one 72th of an inch.
+    point is equal to one 72nd of an inch.
 
     """
 
@@ -109,6 +104,12 @@ class DimensionBase(AcceptNoneAttributeType, metaclass=DimensionType):
         """Evaluate the value of this dimension in points."""
         raise NotImplementedError
 
+    def __eq__(self, other):
+        try:
+            return float(self) == float(other)
+        except (ValueError, TypeError):
+            return False
+
     @classmethod
     def check_type(cls, value):
         return (super().check_type(value) or isinstance(value, Fraction)
@@ -125,7 +126,7 @@ class DimensionBase(AcceptNoneAttributeType, metaclass=DimensionType):
                        """, re.IGNORECASE | re.VERBOSE)
 
     @classmethod
-    def from_tokens(cls, tokens):
+    def from_tokens(cls, tokens, source):
         sign = 1
         if tokens.next.exact_type in (PLUS, MINUS):
             sign = -1 if next(tokens).exact_type == MINUS else 1
@@ -138,16 +139,16 @@ class DimensionBase(AcceptNoneAttributeType, metaclass=DimensionType):
             value = float(token.string)
         if tokens.next and tokens.next.type in (NAME, OP):
             unit_string = next(tokens).string
-        elif value != 0:
-            raise ParseError('Expecting a dimension unit')
+        elif value == 0:
+            return Dimension(0)
         else:
-            return value
+            raise ParseError('Expecting a dimension unit')
         if unit_string == '/':
             unit_string += next(tokens).string
         try:
             unit = DimensionUnitBase.all[unit_string.lower()]
         except KeyError:
-            raise ValueError("'{}' is not a valid dimension unit"
+            raise ParseError("'{}' is not a valid dimension unit"
                              .format(unit_string))
         return sign * value * unit
 
