@@ -32,13 +32,14 @@ from copy import copy
 from itertools import count
 
 from . import __version__, __release_date__
-from .attribute import OptionSet
+from .attribute import OptionSet, Source
 from .backend import pdf
 from .flowable import StaticGroupedFlowables
 from .language import EN
 from .layout import (Container, ReflowRequired,
                      BACKGROUND, CONTENT, HEADER_FOOTER)
 from .number import format_number
+from .strings import Strings
 from .style import StyleLog
 from .util import RefKeyDictionary
 from .warnings import warn
@@ -52,21 +53,16 @@ class DocumentTree(StaticGroupedFlowables):
 
     Args:
         flowables (list[Flowable]): the list of top-level flowables
-        source_file (Path): absolute path of the source file, used to locate
-            images and include in logging and error and warnings.
+        source_root (Path): path used to locate images and include in logging
+            and error and warnings.
         options (Reader): frontend-specific options
 
     """
 
-    def __init__(self, flowables, source_file=None, options=None):
+    def __init__(self, flowables, options=None):
         super().__init__(flowables)
-        self.source_file = source_file.absolute() if source_file else None
         self.options = options
 
-    @property
-    def source_root(self):
-        return self.source_file.parent if self.source_file else Path.cwd()
-
 
 class PageOrientation(OptionSet):
     values = 'portrait', 'landscape'
@@ -169,21 +165,13 @@ class Page(Container):
         self.canvas.place_annotations()
 
 
-class BackendDocumentMetadata(object):
-    def __init__(self, name):
-        self.name = name
-
-    def __get__(self, instance, object_type):
-        return instance.backend_document.get_metadata(self.name)
-
-    def __set__(self, instance, value):
-        return instance.backend_document.set_metadata(self.name, value)
-
-
 class PartPageCount(object):
     def __init__(self):
         self.count = 0
 
+    def __repr__(self):
+        return repr(self.count)
+
     def __eq__(self, other):
         return self.count == other.count
 
@@ -192,6 +180,25 @@ class PartPageCount(object):
         return self
 
 
+class Metadata(dict, Source):
+    def __init__(self, document, **items):
+        super().__init__(**items)
+        self._document = document
+
+    def __setitem__(self, key, value):
+        try:
+            value.source = self
+        except AttributeError:
+            pass
+        super().__setitem__(key, value)
+
+    def __getitem__(self, key):
+        return copy(super().__getitem__(key))
+
+    @property
+    def location(self):
+        return 'document metadata'
+
 
 class Document(object):
     """Renders a document tree to pages
@@ -209,28 +216,22 @@ class Document(object):
 
     CACHE_EXTENSION = '.rtc'
 
-    # FIXME: get backend document metadata from Document metadata
-    title = BackendDocumentMetadata('title')
-    author = BackendDocumentMetadata('author')
-    subject = BackendDocumentMetadata('subject')
-    keywords = BackendDocumentMetadata('keywords')
-
     def __init__(self, document_tree, stylesheet, language, strings=None,
                  backend=None):
         """`backend` specifies the backend to use for rendering the document."""
         super().__init__()
         self._print_version_and_license()
         self.front_matter = []
+        self.supporting_matter = {}
         self.document_tree = document_tree
         self.stylesheet = stylesheet
         self.language = language
-        self._strings = strings or ()
+        self._strings = strings or Strings()
         self.backend = backend or pdf
-        self.backend_document = self.backend.Document(self.CREATOR)
         self._flowables = list(id(element)
                                for element in document_tree.elements)
 
-        self.metadata = dict(date=datetime.date.today())
+        self.metadata = Metadata(self, date=datetime.date.today())
         self.counters = {}             # counters for Headings, Figures, Tables
         self.elements = OrderedDict()  # mapping id's to flowables
         self.ids_by_element = RefKeyDictionary()    # mapping elements to id's
@@ -239,12 +240,14 @@ class Document(object):
         self.page_references = {}      # mapping id's to page numbers
         self._sections = []
         self.index_entries = {}
+        self._glossary = {}
+        self._glossary_first = {}
         self._unique_id = 0
         self.error = False
 
     def _print_version_and_license(self):
-        print('rinohtype {} ({})  Copyright (c) Brecht Machiels'
-              .format(__version__, __release_date__))
+        print('rinohtype {} ({})  Copyright (c) Brecht Machiels and'
+              ' contributors'.format(__version__, __release_date__))
         print('''\
 This program comes with ABSOLUTELY NO WARRANTY. Its use is subject
 to the terms of the GNU Affero General Public License version 3.''')
@@ -257,7 +260,7 @@ to the terms of the GNU Affero General Public License version 3.''')
         return self._unique_id
 
     def get_metadata(self, key):
-        return copy(self.metadata.get(key))
+        return self.metadata.get(key)
 
     def register_element(self, element):
         primary_id = (element.get_id(self, create=False)
@@ -280,22 +283,43 @@ to the terms of the GNU Affero General Public License version 3.''')
     def get_reference(self, id, reference_type):
         return self.references[id][reference_type]
 
+    def set_glossary(self, term, definition):
+        try:
+            existing_definition = self._glossary[term]
+            return definition == existing_definition
+        except KeyError:
+            self._glossary[term] = definition
+            return True
+
+    def get_glossary(self, term, id):
+        try:
+            first_id = self._glossary_first[term]
+        except KeyError:
+            self._glossary_first[term] = id
+            return self._glossary[term], id
+        return self._glossary[term], first_id
+
     def _load_cache(self, filename):
         """Load the cached page references from `<filename>.ptc`."""
+        cache_path = filename.with_suffix(self.CACHE_EXTENSION)
         try:
-            with open(filename + self.CACHE_EXTENSION, 'rb') as file:
-                prev_number_of_pages, prev_page_references = pickle.load(file)
+            with cache_path.open('rb') as file:
+                part_page_counts, page_references = pickle.load(file)
+            print('References cache read from {}'.format(cache_path))
         except (IOError, TypeError):
-            prev_number_of_pages, prev_page_references = {}, {}
-        return prev_number_of_pages, prev_page_references
+            part_page_counts, page_references = {}, {}
+        return part_page_counts, page_references
 
-    def _save_cache(self, filename, section_number_of_pages, page_references):
+    def _save_cache(self, filename):
         """Save the current state of the page references to `<filename>.rtc`"""
         cache_path = Path(filename).with_suffix(self.CACHE_EXTENSION)
         with cache_path.open('wb') as file:
-            cache = (section_number_of_pages, page_references)
+            cache = (self.part_page_counts, self.page_references)
             pickle.dump(cache, file)
 
+    def set_string(self, strings_class, key, value):
+        self._strings[strings_class][key] = value
+
     def get_string(self, strings_class, key):
         if (strings_class in self._strings
                 and key in self._strings[strings_class]):
@@ -315,7 +339,7 @@ to the terms of the GNU Affero General Public License version 3.''')
         self.error = False
         filename_root = Path(filename_root) if filename_root else None
         if filename_root and file is None:
-            extension = self.backend_document.extension
+            extension = self.backend.Document.extension
             filename = filename_root.with_suffix(extension)
             file = filename.open('wb')
         elif file and filename_root is None:
@@ -324,39 +348,30 @@ to the terms of the GNU Affero General Public License version 3.''')
             raise ValueError("You need to specify either 'filename_root' or "
                              "'file'.")
 
-        def has_converged(part_page_counts):
-            """Return `True` if the last rendering iteration converged to a
-            stable result.
-
-            Practically, this tests whether the total number of pages and page
-            references to document elements have changed since the previous
-            rendering iteration."""
-            nonlocal prev_number_of_pages, prev_page_references
-            return (part_page_counts == prev_number_of_pages and
-                    self.page_references == prev_page_references)
-
         fake_container = FakeContainer(self)
+        prev_page_counts, prev_page_refs = self._load_cache(filename_root)
+        metadata_args = {key: self.get_metadata(key)
+                         for key in ('title', 'author')}
         try:
             self.document_tree.build_document(fake_container)
-            (prev_number_of_pages,
-             prev_page_references) = self._load_cache(filename_root)
-
-            self.part_page_counts = prev_number_of_pages
             self.prepare(fake_container)
             self.page_elements.clear()
-            self.page_references = prev_page_references.copy()
-            self.part_page_counts = self._render_pages()
-            while not has_converged(self.part_page_counts):
-                prev_number_of_pages = self.part_page_counts
-                prev_page_references = self.page_references.copy()
-                print('Not yet converged, rendering again...')
-                del self.backend_document
-                self.backend_document = self.backend.Document(self.CREATOR)
+            self.part_page_counts = prev_page_counts
+            self.page_references = prev_page_refs.copy()
+            while True:
+                self.backend_document = self.backend.Document(self.CREATOR,
+                                                              **metadata_args)
                 self.part_page_counts = self._render_pages()
-            self.create_outlines()
+                if (self.part_page_counts == prev_page_counts
+                        and self.page_references == prev_page_refs):
+                    break
+                print('Not yet converged, rendering again...')
+                prev_page_counts = self.part_page_counts
+                prev_page_refs = self.page_references.copy()
+                del self.backend_document
+            self._create_outlines(self.backend_document)
             if filename:
-                self._save_cache(filename_root, self.part_page_counts,
-                                 self.page_references)
+                self._save_cache(filename_root)
                 self.style_log.write_log(filename_root)
                 print('Writing output: {}'.format(filename))
             self.backend_document.write(file)
@@ -365,7 +380,30 @@ to the terms of the GNU Affero General Public License version 3.''')
                 file.close()
         return not self.error
 
-    def create_outlines(self):
+    def _render_pages(self):
+        """Render the complete document once and return the number of pages
+        rendered."""
+        self.style_log = StyleLog(self.stylesheet)
+        self.floats = set()
+        self.placed_footnotes = set()
+        self._start_time = time.time()
+
+        part_page_counts = {}
+        part_page_count = PartPageCount()
+        last_number_format = None
+        for part_template in self.part_templates:
+            part = part_template.document_part(self)
+            if part is None:
+                continue
+            if part_template.page_number_format != last_number_format:
+                part_page_count = PartPageCount()
+            part_page_count += part.render(part_page_count.count + 1)
+            part_page_counts[part_template.name] = part_page_count
+            last_number_format = part_template.page_number_format
+        sys.stdout.write('\n')     # for the progress indicator
+        return part_page_counts
+
+    def _create_outlines(self, backend_document):
         """Create an outline in the output file that allows for easy navigation
         of the document. The outline is a hierarchical tree of all the sections
         in the document."""
@@ -394,30 +432,7 @@ to the terms of the GNU Affero General Public License version 3.''')
             item = (str(section_id), section_number, section_title, current)
             parent.append(item)
             current_level = section.level
-        self.backend_document.create_outlines(sections)
-
-    def _render_pages(self):
-        """Render the complete document once and return the number of pages
-        rendered."""
-        self.style_log = StyleLog(self.stylesheet)
-        self.floats = set()
-        self.placed_footnotes = set()
-        self._start_time = time.time()
-
-        part_page_counts = {}
-        part_page_count = PartPageCount()
-        last_number_format = None
-        for part_template in self.part_templates:
-            part = part_template.document_part(self)
-            if part is None:
-                continue
-            if part_template.page_number_format != last_number_format:
-                part_page_count = PartPageCount()
-            part_page_count += part.render(part_page_count.count + 1)
-            part_page_counts[part_template.name] = part_page_count
-            last_number_format = part_template.page_number_format
-        sys.stdout.write('\n')     # for the progress indicator
-        return part_page_counts
+        backend_document.create_outlines(sections)
 
     PROGRESS_TEMPLATE = \
         '\r{:3d}% [{}{}] ETA {:02d}:{:02d} ({:02d}:{:02d}) page {}'
