@@ -18,7 +18,7 @@ that make up the content of a document and are rendered onto its pages.
 
 
 from copy import copy
-from itertools import chain, tee
+from itertools import chain
 from token import NAME
 
 from .attribute import Attribute, OptionSet, Bool, OverrideDefault
@@ -28,21 +28,25 @@ from .draw import ShapeStyle, Rectangle, Line, LineStyle, Stroke
 from .layout import (InlineDownExpandingContainer, VirtualContainer,
                      MaybeContainer, ContainerOverflow, EndOfContainer,
                      PageBreakException, ReflowRequired)
+from .strings import UserStrings
 from .style import Styled, Style
 from .text import StyledText
-from .util import ReadAliasAttribute
+from .util import clamp, ReadAliasAttribute
 
 
-__all__ = ['Flowable', 'FlowableStyle', 'FlowableWidth',
+__all__ = ['Flowable', 'FlowableStyle',
+           'FlowableWidth', 'HorizontalAlignment', 'Break',
            'DummyFlowable', 'AnchorFlowable', 'WarnFlowable',
-           'SetMetadataFlowable', 'AddToFrontMatter',
+           'SetMetadataFlowable', 'SetUserStringFlowable',
+           'SetOutOfLineFlowables',
            'GroupedFlowables', 'StaticGroupedFlowables',
            'LabeledFlowable', 'GroupedLabeledFlowables',
-           'Float',
-           'PageBreak', 'PageBreakStyle', 'Break']
+           'Float', 'PageBreak']
 
 
 class FlowableWidth(OptionSet):
+    """Controls the width of a flowable"""
+
     values = ('auto', 'fill')
 
     @classmethod
@@ -50,25 +54,35 @@ class FlowableWidth(OptionSet):
         return Dimension.check_type(value) or super().check_type(value)
 
     @classmethod
-    def from_tokens(cls, tokens):
+    def from_tokens(cls, tokens, source):
         if tokens.next.type == NAME:
-            return super().from_tokens(tokens)
+            return super().from_tokens(tokens, source)
         else:
-            return Dimension.from_tokens(tokens)
+            return Dimension.from_tokens(tokens, source)
+
+    @classmethod
+    def doc_format(cls):
+        return super(cls, cls).doc_format() + ' or ' + Dimension.doc_format()
 
 
 class HorizontalAlignment(OptionSet):
+    """Controls horizontal placement"""
+
     values = 'left', 'right', 'center'
 
 
+class Break(OptionSet):
+    values = None, 'any', 'left', 'right'
+
+
 class FlowableStyle(Style):
     width = Attribute(FlowableWidth, 'auto', 'Width to render the flowable at')
     horizontal_align = Attribute(HorizontalAlignment, 'left',
                                  'Horizontal alignment of the flowable')
     space_above = Attribute(Dimension, 0, 'Vertical space preceding the '
-                                              'flowable')
+                                          'flowable')
     space_below = Attribute(Dimension, 0, 'Vertical space following the '
-                                              'flowable')
+                                          'flowable')
     margin_left = Attribute(Dimension, 0, 'Left margin')
     margin_right = Attribute(Dimension, 0, 'Right margin')
     padding = Attribute(Dimension, 0, 'Padding')
@@ -85,7 +99,8 @@ class FlowableStyle(Style):
     border_bottom = Attribute(Stroke, None, 'Border below the flowable')
     background_color = Attribute(Color, None, "Color of the area within the "
                                               "flowable's borders")
-    hide = Attribute(Bool, False, 'Suppress rendering the flowable')
+    page_break = Attribute(Break, None, 'Type of page break to insert '
+                                        'before rendering this flowable')
 
 
 class FlowableState(object):
@@ -117,12 +132,13 @@ class Flowable(Styled):
     """
 
     style_class = FlowableStyle
+    break_exception = PageBreakException
 
     def __init__(self, align=None, width=None,
-                 id=None, style=None, parent=None):
+                 id=None, style=None, parent=None, source=None):
         """Initialize this flowable and associate it with the given `style` and
         `parent` (see :class:`Styled`)."""
-        super().__init__(id=id, style=style, parent=parent)
+        super().__init__(id=id, style=style, parent=parent, source=source)
         self.annotation = None
         self.align = align
         self.width = width
@@ -160,7 +176,7 @@ class Flowable(Styled):
             return
         if self._width(container) == FlowableWidth.FILL:
             self.warn("horizontal_align has no effect on flowables for which "
-                      "width is set to 'full'", container)
+                      "width is set to 'fill'", container)
             return
         if align == HorizontalAlignment.CENTER:
             offset = float(container.width - bordered_width) / 2
@@ -168,21 +184,33 @@ class Flowable(Styled):
             offset = float(container.width - bordered_width)
         container.left += offset
 
-    def flow(self, container, last_descender, state=None, **kwargs):
+    def flow(self, container, last_descender, state=None, space_below=0,
+             **kwargs):
         """Flow this flowable into `container` and return the vertical space
         consumed.
 
-        The flowable's contents is preceded by a vertical space with a height
+        The flowable's contents are preceded by a vertical space with a height
         as specified in its style's `space_above` attribute. Similarly, the
         flowed content is followed by a vertical space with a height given
         by the `space_below` style attribute."""
         top_to_baseline = 0
+        page_break = self.get_style('page_break', container)
         state = state or self.initial_state(container)
         if state.initial:
+            if page_break:
+                page_number = container.page.number
+                this_page_type = 'left' if page_number % 2 == 0 else 'right'
+                if not (container.page._empty
+                        and page_break in (Break.ANY, this_page_type)):
+                    if page_break == Break.ANY:
+                        page_break = 'left' if page_number % 2 else 'right'
+                    chain = container.top_level_container.chain
+                    raise self.break_exception(page_break, chain, state)
             space_above = self.get_style('space_above', container)
             if not container.advance2(float(space_above)):
                 raise EndOfContainer(state)
             top_to_baseline += float(space_above)
+        self_space_below = float(self.get_style('space_below', container))
         margin_left = self.get_style('margin_left', container)
         margin_right = self.get_style('margin_right', container)
         reference_id = self.get_id(container.document, create=False)
@@ -193,8 +221,9 @@ class Flowable(Styled):
                                                         right=right)
         initial_before = state.initial
         state, width, inner_top_to_baseline, descender = \
-            self.flow_inner(margin_container, last_descender,
-                            state=state, **kwargs)
+            self.flow_inner(margin_container, last_descender, state=state,
+                            space_below=space_below + self_space_below,
+                            **kwargs)
         self._align(margin_container, width)
         initial_after = state is not None and state.initial
         top_to_baseline += inner_top_to_baseline
@@ -211,8 +240,7 @@ class Flowable(Styled):
         if state is not None:
             raise EndOfContainer(state)
 
-        space_below = self.get_style('space_below', container)
-        container.advance2(float(space_below), ignore_overflow=True)
+        container.advance2(self_space_below, ignore_overflow=True)
         container.document.progress(self, container)
         return margin_left + width + margin_right, top_to_baseline, descender
 
@@ -375,13 +403,27 @@ class SetMetadataFlowable(DummyFlowable):
         flowable_target.document.metadata.update(self.metadata)
 
 
-class AddToFrontMatter(DummyFlowable):
-    def __init__(self, flowables, parent=None):
+class SetUserStringFlowable(DummyFlowable):
+    """Add an entry to the document's :class:`UserStrings`"""
+    def __init__(self, label, content, parent=None):
         super().__init__(parent=parent)
+        self.label = label
+        self.content = content
+
+    def build_document(self, flowable_target):
+        doc = flowable_target.document
+        doc.set_string(UserStrings, self.label, self.content)
+
+
+class SetOutOfLineFlowables(DummyFlowable):
+    def __init__(self, names, flowables, parent=None):
+        super().__init__(parent=parent)
+        self.names = names
         self.flowables = flowables
 
     def build_document(self, flowable_target):
-        flowable_target.document.front_matter.append(self.flowables)
+        for name in self.names:
+            flowable_target.document.supporting_matter[name] = self.flowables
 
 
 # grouping flowables
@@ -398,7 +440,7 @@ class GroupedFlowablesState(FlowableState):
 
     @property
     def at_end(self):
-        return self._index == len(self.flowables) - 1
+        return self._index >= len(self.flowables)
 
     def __copy__(self):
         copy_flowables = copy(self.flowables)
@@ -449,7 +491,7 @@ class GroupedFlowables(Flowable):
         flowables_iter = self.flowables(container)
         title_text = self.get_style('title', container)
         if title_text:
-            title = Paragraph(title_text, style='title')
+            title = Paragraph(title_text.copy(), style='title')
             flowables_iter = chain((title, ), flowables_iter)
         return GroupedFlowablesState(self, flowables_iter)
 
@@ -494,12 +536,12 @@ class GroupedFlowables(Flowable):
         except StopIteration:
             raise LastFlowableException(descender)
         flowable.parent = self
-        space_below = space_below if state.at_end else 0
         with MaybeContainer(container) as maybe_container:
             max_flowable_width, top_to_baseline, descender = \
                 flowable.flow(maybe_container, descender,
                               state=state.first_flowable_state,
-                              space_below=space_below, **kwargs)
+                              space_below=space_below if state.at_end else 0,
+                              **kwargs)
         state.initial = False
         state.first_flowable_state = None
         if flowable.get_style('keep_with_next', container):
@@ -507,7 +549,9 @@ class GroupedFlowables(Flowable):
             maybe_container.advance(item_spacing)
             try:
                 width, _, descender = self._flow_with_next(state, container,
-                                                           descender, **kwargs)
+                                                           descender,
+                                                           space_below=space_below,
+                                                           **kwargs)
             except EndOfContainer as eoc:
                 if eoc.flowable_state.initial:
                     maybe_container.do_place(False)
@@ -535,8 +579,10 @@ class StaticGroupedFlowables(GroupedFlowables):
 
     """
 
-    def __init__(self, flowables, id=None, style=None, parent=None):
-        super().__init__(id=id, style=style, parent=parent)
+    def __init__(self, flowables, align=None, width=None,
+                 id=None, style=None, parent=None, source=None):
+        super().__init__(align=align, width=width,
+                         id=id, style=style, parent=parent, source=source)
         self.children = []
         for flowable in flowables:
             self.append(flowable)
@@ -565,6 +611,10 @@ class StaticGroupedFlowables(GroupedFlowables):
             flowable.parent = self
             flowable.prepare(flowable_target)
 
+    @property
+    def empty(self):
+        return not self.children
+
 
 class LabeledFlowableStyle(FlowableStyle):
     label_min_width = Attribute(Dimension, 12*PT, 'Minimum label width')
@@ -618,46 +668,38 @@ class LabeledFlowable(Flowable):
         self.flowable.prepare(flowable_target)
 
     def label_width(self, container):
+        label_min_width = self.get_style('label_min_width', container)
         label_max_width = self.get_style('label_max_width', container)
         virtual_container = VirtualContainer(container)
         label_width, _, _ = self.label.flow(virtual_container, 0)
         spillover = (label_width > label_max_width.to_points(container.width)
                      if label_max_width else True)
-        return label_width, spillover
+        return max(label_width, label_min_width), spillover
 
     def initial_state(self, container):
         initial_content_state = self.flowable.initial_state(container)
         return LabeledFlowableState(self, initial_content_state)
 
     def render(self, container, last_descender, state, label_column_width=None,
-               **kwargs):
+               space_below=0, **kwargs):
         def style(name):
             return self.get_style(name, container)
 
         label_min_width = style('label_min_width').to_points(container.width)
-        label_max_width = style('label_max_width')
-        if label_max_width:
-            label_max_width = label_max_width.to_points(container.width)
-        label_spacing = style('label_spacing')
-        wrap_label = style('wrap_label')
+        label_max_width_ = style('label_max_width')
+        label_max_width = (label_max_width_.to_points(container.width)
+                           if label_max_width_ else float('+inf'))
         align_baselines = style('align_baselines')
 
         free_label_width, _ = self.label_width(container)
-
         label_spillover = False
-        if label_column_width:  # part of a GroupedLabeledFlowables
-            label_width = label_column_width
-        elif free_label_width < label_min_width:
-            label_width = label_min_width
-        elif label_max_width and free_label_width <= label_max_width:
-            label_width = free_label_width
-        else:
-            label_width = label_min_width
-
-        if not label_max_width:
+        label_width = label_column_width or clamp(label_min_width,
+                                                  free_label_width,
+                                                  label_max_width)
+        if label_max_width is None:
             label_spillover = True
         elif free_label_width > label_width:
-            if wrap_label:
+            if style('wrap_label'):
                 vcontainer = VirtualContainer(container, width=label_max_width)
                 wrapped_width, _, _ = self.label.flow(vcontainer, 0)
                 if wrapped_width < label_max_width:
@@ -668,13 +710,13 @@ class LabeledFlowable(Flowable):
             else:
                 label_spillover = True
 
-        left = label_width + label_spacing
-        label_cntnr_width = None if label_spillover else label_width
+        left = label_width + style('label_spacing')
+        max_label_width = None if label_spillover else label_width
 
         if align_baselines and (state.initial and not label_spillover):
             label_baseline = find_baseline(self.label, container,
                                            last_descender,
-                                           width=label_cntnr_width)
+                                           width=max_label_width)
             content_state_copy = copy(state.content_flowable_state)
             content_baseline = find_baseline(self.flowable, container,
                                              last_descender, left=left,
@@ -688,29 +730,20 @@ class LabeledFlowable(Flowable):
         try:
             with MaybeContainer(container) as maybe_container:
                 if state.initial:
-                    label_container = \
-                        InlineDownExpandingContainer('LABEL', maybe_container,
-                                                     width=label_cntnr_width,
-                                                     advance_parent=False)
-                    label_container.advance(offset_label)
-                    _, _, label_descender = self.label.flow(label_container,
-                                                            last_descender)
-                    label_height = label_container.height
+                    label_height, label_descender = \
+                        self._flow_label(maybe_container, last_descender,
+                                         max_label_width, offset_label,
+                                         space_below)
                     if label_spillover:
                         maybe_container.advance(label_height)
                         last_descender = label_descender
-                else:
+                else:   # label was placed on previous page
                     label_height = label_descender = 0
                 maybe_container.advance(offset_content)
                 rendering_content = True
-                content_container = \
-                    InlineDownExpandingContainer('CONTENT', maybe_container,
-                                                 left=left,
-                                                 advance_parent=False)
-                width, _, content_descender \
-                    = self.flowable.flow(content_container, last_descender,
-                                         state=state.content_flowable_state)
-                content_height = content_container.cursor
+                content_height, content_descender, width = \
+                    self._flow_content(maybe_container, last_descender, state,
+                                       left, space_below)
         except (ContainerOverflow, EndOfContainer) as eoc:
             content_state = eoc.flowable_state if rendering_content else None
             state.update(content_state)
@@ -723,6 +756,27 @@ class LabeledFlowable(Flowable):
             descender = label_descender
         return left + width, label_baseline, descender
 
+    def _flow_label(self, container, last_descender, max_width, y_offset,
+                    space_below):
+        label_container = \
+            InlineDownExpandingContainer('LABEL', container, width=max_width,
+                                         advance_parent=False)
+        label_container.advance(y_offset)
+        _, _, label_descender = \
+            self.label.flow(label_container, last_descender, None, space_below)
+        return label_container.height, label_descender
+
+    def _flow_content(self, container, last_descender, state, left,
+                      space_below):
+        content_container = \
+            InlineDownExpandingContainer('CONTENT', container, left=left,
+                                         advance_parent=False)
+        width, _, content_descender \
+            = self.flowable.flow(content_container, last_descender,
+                                 state=state.content_flowable_state,
+                                 space_below=space_below)
+        return content_container.cursor, content_descender, width
+
 
 def find_baseline(flowable, container, last_descender, state=None, **kwargs):
     virtual_container = VirtualContainer(container)
@@ -755,7 +809,7 @@ class GroupedLabeledFlowables(GroupedFlowables):
             max_label_width = state.max_label_width
         try:
             return super().render(container, descender, state=state,
-                                  label_column_width=max_label_width)
+                                  label_column_width=max_label_width, **kwargs)
         except EndOfContainer as eoc:
             eoc.flowable_state.max_label_width = max_label_width
             raise
@@ -793,43 +847,13 @@ class Float(Flowable):
                                 **kwargs)
 
 
-class Break(OptionSet):
-    values = None, 'any', 'left', 'right'
-
-
-class PageBreakStyle(FlowableStyle):
-    page_break = Attribute(Break, None, 'Type of page break to insert '
-                                        'before rendering this flowable')
-
-
 class PageBreak(Flowable):
-    """A flowable that optionally triggers a page break before rendering.
+    def __init__(self, page_break=Break.ANY):
+        style = FlowableStyle(page_break=page_break)
+        super().__init__(style=style)
 
-    If this flowable's `page_break` style attribute is not ``None``, it breaks
-    to the page of the type indicated by `page_break` before starting
-    rendering.
-
-    """
-
-    style_class = PageBreakStyle
-    exception_class = PageBreakException
-
-    def flow(self, container, last_descender, state=None, **kwargs):
-        state = state or self.initial_state(container)
-        page_number = container.page.number
-        this_page_type = 'left' if page_number % 2 == 0 else 'right'
-        page_break = self.get_style('page_break', container)
-        if state.initial and page_break:
-            if not (container.page._empty
-                    and page_break in (Break.ANY, this_page_type)):
-                if page_break == Break.ANY:
-                    page_break = 'left' if page_number % 2 else 'right'
-                chain = container.top_level_container.chain
-                raise self.exception_class(page_break, chain, state)
-        return super().flow(container, last_descender, state)
-
-    def render(self, container, descender, state, **kwargs):
-        return 0, 0, descender
+    def render(self, container, last_descender, state=None, **kwargs):
+        return 0, 0, last_descender
 
 
 from .paragraph import Paragraph
