@@ -22,7 +22,6 @@ from ...util import intersperse
 # (http://docutils.sourceforge.net/docs/ref/doctree.html)
 # - abbreviation? (not exposed in default reStructuredText; Sphinx has :abbr:)
 # - acronym (not exposed in default reStructuredText?)
-# - math / math_block
 # - pending (should not appear in final doctree?)
 # - substitution_reference (should not appear in final doctree?)
 
@@ -180,8 +179,7 @@ class Topic(DocutilsGroupingNode):
                 return rt.StaticGroupedFlowables(flowables,
                                                  style='table of contents')
             else:
-                toc_id, = self.get('ids')
-                return rt.SetMetadataFlowable(toc_id=toc_id)
+                return rt.SetMetadataFlowable(toc_ids=self.get('ids'))
         elif 'dedication' in classes:
             return self._process_topic('dedication')
         elif 'abstract' in classes:
@@ -233,6 +231,14 @@ class Subtitle(DocutilsBodyNode):
         return rt.SetMetadataFlowable(subtitle=self.process_content())
 
 
+class Math_Block(DocutilsBodyNode):
+    style = 'math'
+
+    def build_flowables(self):
+        yield rt.WarnFlowable("The 'math' directive is not yet supported")
+        yield rt.Paragraph(self.process_content(), style=self.style)
+
+
 class Admonition(DocutilsGroupingNode):
     grouped_flowables_class = rt.Admonition
 
@@ -312,6 +318,14 @@ class Literal(Inline):
     style = 'monospaced'
 
 
+class Math(Inline):
+    style = 'math'
+
+    def build_styled_text(self):
+        return (rt.WarnInline("The 'math' role is not yet supported")
+                + super().build_styled_text())
+
+
 class Superscript(DocutilsInlineNode):
     def build_styled_text(self):
         return rt.Superscript(self.process_content())
@@ -439,7 +453,9 @@ class Citation_Reference(Footnote_Reference):
 
 class Substitution_Definition(DocutilsBodyNode):
     def build_flowable(self):
-        return rt.DummyFlowable()
+        label, = self.node.attributes['names']
+        content = self.process_content()
+        return rt.SetUserStringFlowable(label, content)
 
 
 class Target(DocutilsBodyNode, DocutilsInlineNode):
@@ -455,24 +471,29 @@ class Target(DocutilsBodyNode, DocutilsInlineNode):
         return rt.AnchorFlowable()
 
 
-class Enumerated_List(DocutilsBodyNode):
+class Enumerated_List(DocutilsGroupingNode):
+    style = 'enumerated'
+    grouped_flowables_class = rt.List
+
     def build_flowable(self):
         # TODO: handle different numbering styles
-        return rt.List([item.flowable() for item in self.list_item],
-                       style='enumerated')
+        start = self.attributes.get('start', 1)
+        return super().build_flowable(start_index=start)
 
+class Bullet_List(DocutilsGroupingNode):
+    style = 'bulleted'
+    grouped_flowables_class = rt.List
 
-class Bullet_List(DocutilsBodyNode):
     def build_flowable(self):
         try:
-            return rt.List([item.flowable() for item in self.list_item],
-                           style='bulleted')
+            return super().build_flowable()
         except AttributeError:  # empty list
             return rt.DummyFlowable()
 
 
 class List_Item(DocutilsGroupingNode):
-    pass
+    def build_flowable(self):
+        return rt.ListItem(super().build_flowable())
 
 
 class Definition_List(DocutilsGroupingNode):
@@ -518,7 +539,7 @@ class Field_List(DocutilsGroupingNode):
 class Field(DocutilsBodyNode):
     @property
     def name(self):
-        return str(self.field_name.styled_text())
+        return self.field_name.text
 
     @property
     def value(self):
@@ -653,14 +674,22 @@ class Table(DocutilsBodyNode):
         except AttributeError:
             head = None
         body = tgroup.tbody.get_table_section()
+        align = self.get('align')
         width_string = self.get('width')
-        table = rt.Table(body, head=head, width=convert_quantity(width_string),
-                         column_widths=column_widths)
+        return rt.Table(body, head=head,
+                        align=None if align == 'default' else align,
+                        width=convert_quantity(width_string),
+                        column_widths=column_widths)
+
+    def flowables(self):
+        table, = super().flowables()
         try:
             caption = rt.Caption(self.title.process_content())
-            return rt.TableWithCaption([caption, table])
+            table_with_caption = rt.TableWithCaption([caption, table])
+            table_with_caption.classes.extend(self.get('classes'))
+            yield table_with_caption
         except AttributeError:
-            return table
+            yield table
 
 
 class TGroup(DocutilsNode):
@@ -702,9 +731,8 @@ class Entry(DocutilsGroupingNode):
 
 class Raw(DocutilsBodyNode, DocutilsInlineNode):
     def build_styled_text(self):
-        cls, = self['classes']
-        return rt.WarnInline('{}: raw interpreted text roles are not supported'
-                             .format(cls))
+        if self['format'] == 'rinoh':
+            return rt.StyledText.from_string(self.text)
 
     def build_flowable(self):
         if self['format'] == 'rinoh':
@@ -721,12 +749,31 @@ class Raw(DocutilsBodyNode, DocutilsInlineNode):
                                    .format(self.text))
         elif self['format'] == 'pdf':   # rst2pdf
             if self.text == 'PageBreak':
-                style = rt.PageBreakStyle(page_break=rt.Break.ANY)
-                return rt.PageBreak(style=style)
+                return rt.PageBreak()
             return rt.WarnFlowable("Unsupported raw pdf option: '{}'"
                                    .format(self.text))
         return rt.DummyFlowable()
 
 
 class Container(DocutilsGroupingNode):
+    @property
+    def set_id(self):
+        return 'out-of-line' not in self['classes']
+
+    def build_flowable(self, style=None, **kwargs):
+        classes = self.get('classes')
+        if 'literal-block-wrapper' in classes:
+            return rt.CodeBlockWithCaption(self.children_flowables(),
+                                           style=style or self.style, **kwargs)
+        if 'out-of-line' in classes:
+            names = self['names']
+            if not names:
+                raise MissingName('out-of-line container is missing a :name:'
+                                  ' to reference it by')
+            return rt.SetOutOfLineFlowables(names, self.children_flowables(),
+                                            **kwargs)
+        return super().build_flowable(style, **kwargs)
+
+
+class MissingName(Exception):
     pass
