@@ -8,6 +8,8 @@
 from distutils.version import LooseVersion
 
 from .color import HexColor
+from .flowable import (StaticGroupedFlowables, GroupedFlowablesStyle,
+                       Float, FloatStyle)
 from .font.style import FontWeight, FontSlant
 from .paragraph import Paragraph
 from .style import StyledMatcher, StyleSheet
@@ -32,7 +34,8 @@ except ImportError:
     PYGMENTS_AVAILABLE = False
 
 
-__all__ = ['CodeBlock', 'Token', 'pygments_style_to_stylesheet']
+__all__ = ['CodeBlock', 'CodeBlockWithCaption', 'Token',
+           'pygments_style_to_stylesheet']
 
 
 class CodeBlock(Paragraph):
@@ -50,6 +53,15 @@ class CodeBlock(Paragraph):
         super().__init__(text, id=id, style=style, parent=parent)
 
 
+class CodeBlockWithCaptionStyle(FloatStyle, GroupedFlowablesStyle):
+    pass
+
+
+class CodeBlockWithCaption(Float, StaticGroupedFlowables):
+    style_class = CodeBlockWithCaptionStyle
+    category = 'Listing'
+
+
 def highlight_block(language, text, lexer_getter):
     get_lexer = lexer_getter or (lambda text, lang: get_lexer_by_name(lang))
     lexer = get_lexer(text, language)
