@@ -11,6 +11,7 @@ Functions for formatting numbers:
 * :func:`format_number`: Format a number according to a given style.
 
 """
+from copy import copy
 
 from .attribute import Attribute, OptionSet, Bool
 from .paragraph import ParagraphBase, ParagraphStyle
@@ -18,10 +19,13 @@ from .style import Style
 from .text import StyledText
 
 
-__all__ = ['NumberStyle', 'Label', 'NumberedParagraph', 'format_number']
+__all__ = ['NumberFormat', 'NumberStyle', 'Label', 'NumberedParagraph',
+           'format_number']
 
 
 class NumberFormat(OptionSet):
+    """How (or if) numbers are displayed"""
+
     values = (None, 'number', 'symbol', 'lowercase character',
               'uppercase character', 'lowercase roman', 'uppercase roman')
 
@@ -97,7 +101,7 @@ class Label(object):
     def format_label(self, label, container):
         prefix = self.get_style('label_prefix', container) or ''
         suffix = self.get_style('label_suffix', container) or ''
-        return prefix + label + suffix
+        return copy(prefix) + copy(label) + copy(suffix)
 
 
 class NumberStyle(LabelStyle):
