@@ -17,7 +17,6 @@ from ast import literal_eval
 from copy import copy
 from functools import partial
 from itertools import tee, chain, groupby, count
-from math import floor
 from os import path
 
 from . import DATA_PATH
@@ -42,6 +41,8 @@ __all__ = ['Paragraph', 'ParagraphStyle', 'TabStop',
 # Text justification
 
 class TextAlign(OptionSet):
+    """Text justification"""
+
     values = 'left', 'right', 'center', 'justify'
 
 
@@ -74,7 +75,7 @@ class LineSpacing(AttributeType):
     REGEX = re.compile(r'^(?P<type>[a-z]+)(\((?P<arg>.*)\))?$', re.I)
 
     @classmethod
-    def parse_string(cls, string):
+    def parse_string(cls, string, source):
         m = cls.REGEX.match(string)
         if not m:
             raise ParseError("'{}' is not a valid {} type"
@@ -249,19 +250,23 @@ PREDEFINED_SPACINGS = dict(default=DEFAULT,
 
 
 class TabAlign(OptionSet):
+    """Alignment of text with respect to a tab stop"""
+
     values = 'left', 'right', 'center'
 
 
 class TabStop(object):
-    """Horizontal position for aligning text of successive lines."""
+    """Horizontal position for aligning text of successive lines
+
+    Args:
+        position (:class:`Dimension` or :class:`Fraction`): tab stop position
+        align (TabAlign): the alignment of text with respect to the tab stop
+            positon
+        fill (str): string pattern to fill the empty tab space with
+
+    """
 
     def __init__(self, position, align='left', fill=None):
-        """`position` can be an absolute position (:class:`Dimension`) or can
-        be relative to the line width (:class:`Fraction`).
-        The alignment of text with respect to the tab stop is determined by
-        `align`, which can be :const:`LEFT`, :const:`RIGHT` or :const:`CENTER`.
-        Optionally, `fill` specifies a string pattern to fill the empty tab
-        space with."""
         self._position = position
         self.align = align
         self.fill = fill
@@ -283,6 +288,8 @@ class TabStop(object):
 
 
 class TabStopList(AttributeType, list):
+    """List of tab stop positions (with alignment and fill string)"""
+
     def __str__(self):
         return ', '.join(str(tab_stop) for tab_stop in self)
 
@@ -312,7 +319,7 @@ class TabStopList(AttributeType, list):
                        re.IGNORECASE | re.VERBOSE)
 
     @classmethod
-    def parse_string(cls, string):
+    def parse_string(cls, string, source):
         tabstops = []
         i = 0
         while string[i:]:
@@ -332,7 +339,7 @@ class TabStopList(AttributeType, list):
     def doc_format(cls):
         return ('a comma-seperated list of tab stops. A tab stop is specified '
                 'as ``<position> [align] [fill string]``, where '
-                '``<position>`` (:class:`.Dimension`) is required and '
+                '``position`` (:class:`.Dimension`) is required and '
                 '``align`` (:class:`.TabAlign`) and ``fill string`` '
                 '(string enclosed in quotes) are optional.')
 
@@ -592,7 +599,7 @@ class ParagraphBase(Flowable):
         return self
 
     def initial_state(self, container):
-        spans = self.text(container).spans(container)
+        spans = self.text(container).wrapped_spans(container)
         return ParagraphState(self, spans_to_words(spans, container))
 
     def _short_repr_args(self, flowable_target):
