@@ -18,6 +18,7 @@ from .layout import ReflowRequired, ContainerOverflow
 from .number import NumberStyle, Label, format_number
 from .paragraph import Paragraph, ParagraphStyle, ParagraphBase
 from .strings import StringCollection, StringField
+from .style import HasClass, HasClasses
 from .text import (SingleStyledTextBase, MixedStyledTextBase, TextStyle,
                    StyledText, SingleStyledText, MixedStyledText)
 from .util import NotImplementedAttribute
@@ -44,8 +45,8 @@ class ReferenceType(OptionSet):
 
 class ReferenceBase(MixedStyledTextBase):
     def __init__(self, type='number', link=True, quiet=False, style=None,
-                 parent=None):
-        super().__init__(style=style, parent=parent)
+                 parent=None, source=None):
+        super().__init__(style=style, parent=parent, source=source)
         self.type = type
         self.link = link
         self.quiet = quiet
@@ -56,13 +57,13 @@ class ReferenceBase(MixedStyledTextBase):
             result += ' ({})'.format(self.style)
         return result
 
+    def copy(self, parent=None):
+        return type(self)(self.type, self.link, self.quiet,
+                          style=self.style, parent=parent)
+
     def target_id(self, document):
         raise NotImplementedError
 
-    @property
-    def items(self):
-        return [self]
-
     def children(self, container):
         if container is None:
             return '$REF({})'.format(self.type)
@@ -149,13 +150,14 @@ class ReferenceText(StyledText):
 
     @classmethod
     def _substitute_variables(cls, text, style):
-        def create_reference_field(key, style=None):
-            return ReferenceField(key.lower(), style=style)
-
         return substitute_variables(text, cls.RE_TYPES, create_reference_field,
                                     super()._substitute_variables, style)
 
 
+def create_reference_field(key, style=None):
+    return ReferenceField(key.lower(), style=style)
+
+
 class ReferencingParagraphStyle(ParagraphStyle):
     text = Attribute(ReferenceText, ReferenceField('title'),
                      'The text content of this paragraph')
@@ -169,8 +171,28 @@ class ReferencingParagraph(ParagraphBase):
         self.target_id_or_flowable = target_id_or_flowable
 
     def text(self, container):
-        return MixedStyledText(self.get_style('text', container), parent=self)
+        return self.get_style('text', container).copy(parent=self)
 
+    def _target_id(self, document):
+        target_id_or_flowable = self.target_id_or_flowable
+        return (target_id_or_flowable.get_id(document)
+                if isinstance(target_id_or_flowable, Flowable)
+                else target_id_or_flowable)
+
+    def _target_flowable(self, document):
+        return document.elements[self._target_id(document)]
+
+    def target_style(self, document):
+        """Filter selection on the ``style`` attribute of the target flowable"""
+        return self._target_flowable(document).style
+
+    def target_has_class(self, document):
+        """Filter selection on a class of the target flowable"""
+        return HasClass(self._target_flowable(document))
+
+    def target_has_classes(self, document):
+        """Filter selection on a set of classes of the target flowable"""
+        return HasClasses(self._target_flowable(document))
 
 
 class Note(LabeledFlowable):
@@ -319,11 +341,9 @@ class SECTION_TITLE(SectionFieldType):
 
 from . import structure    # fills StringCollection.subclasses
 
-RE_STRINGFIELD = ('|'.join(r'{}\.(?:{})'
-                           .format(collection_name, '|'.join(s.name for s
-                                                             in cls._strings))
-                           for collection_name, cls
-                           in StringCollection.subclasses.items()))
+RE_STRINGFIELD = ('|'.join(r'{}\.(?:[a-z_][a-z0-9_]*)'
+                           .format(collection_name)
+                           for collection_name in StringCollection.subclasses))
 
 
 class Field(MixedStyledTextBase):
@@ -348,10 +368,6 @@ class Field(MixedStyledTextBase):
             field = SectionFieldType.from_string(string)
         return cls(field, style=style)
 
-    @property
-    def items(self):
-        return [self]
-
     def children(self, container):
         if container is None:
             text = '${}'.format(self.type)
