@@ -9,7 +9,7 @@
 from itertools import chain
 
 from .attribute import AcceptNoneAttributeType
-from .text import StyledText, SingleStyledTextBase
+from .text import StyledText, SingleStyledText, MixedStyledTextBase
 from .util import NamedDescriptor, WithNamedDescriptors
 
 
@@ -81,6 +81,21 @@ class StringCollection(dict, metaclass=StringCollectionMeta):
         return getattr(self, name)
 
 
+class UserStrings(dict, metaclass=StringCollectionMeta):
+    """Collection of user-specified strings
+
+    Unlike other string collections, these are not limited to a predefined set.
+
+    """
+
+    def __init__(self, **strings):
+        for name, value in strings.items():
+            if not StyledText.check_type(value):
+                raise TypeError('String attributes only accept styled text'
+                                .format(self.name))
+            self[name] = value
+
+
 class Strings(AcceptNoneAttributeType, dict):
     """Stores several :class:`StringCollection`\\ s"""
 
@@ -95,25 +110,29 @@ class Strings(AcceptNoneAttributeType, dict):
                                      type(self).__name__))
         super().__setitem__(string_collection_class, string_collection)
 
+    def __missing__(self, string_collection_class):
+        instance = string_collection_class()
+        self[string_collection_class] = instance
+        return instance
+
     @classmethod
     def doc_format(cls):
         return ('strings need to be entered in INI sections named after the '
                 ':class:`.StringCollection` subclasses')
 
 
-class StringField(SingleStyledTextBase):
+class StringField(MixedStyledTextBase):
     """Styled text that will be substituted with a configured string
 
-    The configured string is either the localized string as determined by the
+    The displayed string is either the localized string as determined by the
     language set for the document or the user-supplied string passed to the
-    :class:`TemplateConfiguration`
+    :class:`.TemplateConfiguration`.
 
     """
-    def __init__(self, strings_class, key, case=None, style=None, parent=None):
+    def __init__(self, strings_class, key, style=None, parent=None):
         super().__init__(style=style, parent=parent)
         self.strings_class = strings_class
         self.key = key
-        self.case = case
 
     def __eq__(self, other):
         return type(self) == type(other) and self.__dict__ == other.__dict__
@@ -125,39 +144,19 @@ class StringField(SingleStyledTextBase):
         return result
 
     def __repr__(self):
-        return "{}({}, '{}')".format(type(self).__name__,
-                                     self.strings_class, self.key)
+        return ("{}({}, '{}', style={})"
+                .format(type(self).__name__, self.strings_class.__name__,
+                        self.key, self.style))
 
     @classmethod
     def parse_string(cls, string, style=None):
         collection, key = string.split('.')
         return cls(StringCollection.subclasses[collection], key, style=style)
 
-    def string(self, document):
-        return document.get_string(self.strings_class, self.key)
-
-    def text(self, flowable_target):
-        if flowable_target is None:
-            return repr(self)
-        string = self.string(flowable_target.document)
-        try:
-            string = string.to_string(flowable_target)
-        except AttributeError:
-            pass
-        return self.case(string) if self.case else string
-
-    def _case(self, case_function):
-        return type(self)(self.strings_class, self.key, case=case_function,
-                          style=self.style, parent=self.parent)
-
-    def lower(self):
-        return self._case(str.lower)
-
-    def upper(self):
-        return self._case(str.upper)
-
-    def capitalize(self):
-        return self._case(str.capitalize)
-
-    def title(self):
-        return self._case(str.title)
+    def children(self, container):
+        text = container.document.get_string(self.strings_class, self.key)
+        if isinstance(text, StyledText):
+            text.parent = self
+            yield text
+        else:
+            yield SingleStyledText(text, parent=self)
