@@ -6,13 +6,12 @@
 # Public License v3. See the LICENSE file or http://www.gnu.org/licenses/.
 
 
-from itertools import chain
+from itertools import chain, takewhile
 
 from .attribute import Attribute, Bool, Integer, OverrideDefault
 from .draw import Line, LineStyle
 from .element import create_destination
 from .flowable import GroupedFlowables, StaticGroupedFlowables
-from .flowable import PageBreak, PageBreakStyle
 from .flowable import LabeledFlowable, GroupedLabeledFlowables
 from .flowable import Flowable, FlowableStyle, GroupedFlowablesStyle
 from .layout import PageBreakException
@@ -25,7 +24,7 @@ from .reference import ReferenceType
 from .text import StyledText, SingleStyledText, MixedStyledText, Tab
 from .style import PARENT_STYLE
 from .strings import StringCollection, String, StringField
-from .util import NotImplementedAttribute
+from .util import NotImplementedAttribute, itemcount
 
 
 __all__ = ['Section', 'Heading',
@@ -34,7 +33,7 @@ __all__ = ['Section', 'Heading',
            'TableOfContentsSection', 'TableOfContentsStyle', 'TableOfContents',
            'ListOfStyle',
            'TableOfContentsEntry', 'Admonition', 'AdmonitionStyle',
-           'HorizontalRule', 'HorizontalRuleStyle']
+           'HorizontalRule', 'HorizontalRuleStyle', 'OutOfLineFlowables']
 
 
 class SectionTitles(StringCollection):
@@ -47,7 +46,7 @@ class SectionTitles(StringCollection):
     index = String('Title for the index section')
 
 
-class SectionStyle(GroupedFlowablesStyle, PageBreakStyle):
+class SectionStyle(GroupedFlowablesStyle):
     show_in_toc = Attribute(Bool, True, 'List this section in the table of '
                                         'contents')
 
@@ -56,16 +55,9 @@ class NewChapterException(PageBreakException):
     pass
 
 
-class Section(StaticGroupedFlowables, PageBreak):
-    """A subdivision of a document
-
-    A section usually has a heading associated with it, which is optionally
-    numbered.
-
-    """
-
+class SectionBase(GroupedFlowables):
     style_class = SectionStyle
-    exception_class = NewChapterException
+    break_exception = NewChapterException
 
     @property
     def category(self):
@@ -94,6 +86,15 @@ class Section(StaticGroupedFlowables, PageBreak):
         pass    # destination is set by the section's Heading
 
 
+class Section(StaticGroupedFlowables, SectionBase):
+    """A subdivision of a document
+
+    A section usually has a heading associated with it, which is optionally
+    numbered.
+
+    """
+
+
 class HeadingStyle(NumberedParagraphStyle):
     number_separator = Attribute(StyledText, '.',
                                  "Characters inserted between the number "
@@ -184,18 +185,23 @@ class ListStyle(GroupedFlowablesStyle, NumberStyle):
                        'Bullet to use in unordered lists')
 
 
-class List(StaticGroupedFlowables, Label):
+class List(GroupedLabeledFlowables, StaticGroupedFlowables):
     style_class = ListStyle
 
-    def __init__(self, flowables, id=None, style=None, parent=None):
-        items = [ListItem(i + 1, flowable, parent=self)
-                 for i, flowable in enumerate(flowables)]
-        super().__init__(items, id=id, style=style, parent=parent)
+    def __init__(self, list_items, start_index=1,
+                 id=None, style=None, parent=None):
+        super().__init__(list_items, id=id, style=style, parent=parent)
+        self.start_index = start_index
+
+    def index(self, item, container):
+        items = filter(lambda itm: not itm.label.get_style('hide', container),
+                       takewhile(lambda li: li != item, self.children))
+        return self.start_index + itemcount(items)
 
 
 class ListItem(LabeledFlowable):
-    def __init__(self, index, flowable, id=None, style=None, parent=None):
-        label = ListItemLabel(index)
+    def __init__(self, flowable, id=None, style=None, parent=None):
+        label = ListItemLabel()
         super().__init__(label, flowable, id=id, style=style, parent=parent)
 
 
@@ -206,15 +212,13 @@ class ListItemLabelStyle(ParagraphStyle, LabelStyle):
 class ListItemLabel(ParagraphBase, Label):
     style_class = ListItemLabelStyle
 
-    def __init__(self, index, style=None, parent=None):
-        super().__init__(style=style, parent=parent)
-        self.index = index
-
     def text(self, container):
-        list = self.parent.parent
+        list_item = self.parent
+        list = list_item.parent
         if list.get_style('ordered', container):
+            index = list.index(list_item, container)
             number_format = list.get_style('number_format', container)
-            label = format_number(self.index, number_format)
+            label = format_number(index, number_format)
         else:
             label = list.get_style('bullet', container)
         return MixedStyledText(self.format_label(label, container), parent=self)
@@ -252,10 +256,14 @@ class TableOfContentsSection(Section):
 
     def get_id(self, document, create=True):
         try:
-            return document.metadata['toc_id']
+            return document.metadata['toc_ids'][0]
         except KeyError:
             return super().get_id(document, create)
 
+    def get_ids(self, document):
+        yield self.get_id(document)
+        yield from document.metadata.get('toc_ids', [])[1:]
+
 
 class TableOfContents(GroupedFlowables):
     style_class = TableOfContentsStyle
@@ -277,11 +285,13 @@ class TableOfContents(GroupedFlowables):
             while next(items) is not section:  # fast-forward `items` to the
                 pass                           # first sub-section of `section`
             for item in items:
-                if item.level == section.level:
+                if item.level <= section.level:
                     break
                 yield item
 
         depth = self.get_style('depth', container)
+        if self.local and self.section:
+            depth += self.level - 1
         items = (section for section in container.document._sections
                  if section.show_in_toc(container) and section.level <= depth)
         if self.local and self.section:
@@ -461,3 +471,14 @@ class HorizontalRule(Flowable):
         line = Line((0, 0), (width, 0), style=PARENT_STYLE, parent=self)
         line.render(container)
         return width, 0, 0
+
+
+class OutOfLineFlowables(GroupedFlowables):
+    def __init__(self, name, align=None, width=None, id=None, style=None,
+                 parent=None):
+        super().__init__(align=align, width=width, id=id, style=style,
+                         parent=parent)
+        self.name = name
+
+    def flowables(self, container):
+        return container.document.supporting_matter[self.name]
