@@ -22,21 +22,23 @@ import string
 
 from ast import literal_eval
 from collections import OrderedDict, namedtuple
+from contextlib import suppress
 from itertools import chain
 from operator import attrgetter
 from pathlib import Path
 
 from .attribute import (WithAttributes, AttributesDictionary,
                         RuleSet, RuleSetFile, Configurable,
-                        DefaultValueException)
+                        DefaultValueException, Attribute, Bool)
 from .element import DocumentElement
-from .resource import Resource
+from .resource import Resource, ResourceNotFound
 from .util import (cached, all_subclasses, NotImplementedAttribute,
                    class_property)
 from .warnings import warn
 
 
-__all__ = ['Style', 'Styled', 'StyledMatcher', 'StyleSheet', 'StyleSheetFile',
+__all__ = ['Style', 'Styled', 'StyledMeta',
+           'StyledMatcher', 'StyleSheet', 'StyleSheetFile',
            'ClassSelector', 'ContextSelector', 'PARENT_STYLE']
 
 
@@ -59,6 +61,8 @@ class Style(AttributesDictionary, metaclass=StyleMeta):
 
     """
 
+    hide = Attribute(Bool, False, 'Suppress rendering this element')
+
     def __init__(self, base=None, **attributes):
         """Style attributes are as passed as keyword arguments. Supported
         attributes are the :class:`Attribute` class attributes of this style
@@ -128,7 +132,7 @@ class Selector(object):
         return self.pri(-1)
 
     def __eq__(self, other):
-        return self.__dict__ == other.__dict__
+        return type(self) == type(other) and self.__dict__ == other.__dict__
 
     def pri(self, priority):
         return SelectorWithPriority(self, priority)
@@ -146,7 +150,7 @@ class Selector(object):
     def flatten(self, stylesheet):
         raise NotImplementedError
 
-    def match(self, styled, stylesheet):
+    def match(self, styled, stylesheet, document):
         raise NotImplementedError
 
 
@@ -176,8 +180,8 @@ class SelectorWithPriority(Selector):
         flattened_selector = self.selector.flatten(stylesheet)
         return flattened_selector.pri(self.priority)
 
-    def match(self, styled, stylesheet):
-        score = self.selector.match(styled, stylesheet)
+    def match(self, styled, stylesheet, document):
+        score = self.selector.match(styled, stylesheet, document)
         if score:
             score = Specificity(self.priority, 0, 0, 0, 0) + score
         return score
@@ -222,9 +226,9 @@ class SelectorByName(SingleSelector):
         selector = matcher.by_name[self.name]
         return selector.get_style_name(matcher)
 
-    def match(self, styled, stylesheet):
+    def match(self, styled, stylesheet, document):
         selector = stylesheet.get_selector(self.name)
-        return selector.match(styled, stylesheet)
+        return selector.match(styled, stylesheet, document)
 
 
 class ClassSelectorBase(SingleSelector):
@@ -242,7 +246,7 @@ class ClassSelectorBase(SingleSelector):
     def get_style_name(self, matcher):
         return self.style_name
 
-    def match(self, styled, document):
+    def match(self, styled, stylesheet, document):
         if not isinstance(styled, self.cls):
             return None
         class_match = 2 if type(styled) == self.cls else 1
@@ -255,7 +259,12 @@ class ClassSelectorBase(SingleSelector):
 
         attributes_match = 0
         for attr, value in self.attributes.items():
-            if not hasattr(styled, attr) or getattr(styled, attr) != value:
+            if not hasattr(styled, attr):
+                return None
+            attr = getattr(styled, attr)
+            if callable(attr):
+                attr = attr(document)
+            if attr != value:
                 return None
             attributes_match += 1
 
@@ -292,7 +301,7 @@ class ContextSelector(Selector):
     def get_style_name(self, matcher):
         return self.selectors[-1].get_style_name(matcher)
 
-    def match(self, styled, stylesheet):
+    def match(self, styled, stylesheet, document):
         def styled_and_parents(element):
             while element is not None:
                 yield element
@@ -307,13 +316,13 @@ class ContextSelector(Selector):
                 element = next(elements)                # NoMoreParentElement
                 if isinstance(selector, EllipsisSelector):
                     selector = next(selectors)          # StopIteration
-                    while not selector.match(element, stylesheet):
+                    while not selector.match(element, stylesheet, document):
                         element = next(elements)        # NoMoreParentElement
             except NoMoreParentElement:
                 return None
             except StopIteration:
                 break
-            score = selector.match(element, stylesheet)
+            score = selector.match(element, stylesheet, document)
             if not score:
                 return None
             total_score += score
@@ -390,13 +399,13 @@ class Styled(DocumentElement, Configurable, metaclass=StyledMeta):
     """The :class:`Style` subclass that corresponds to this :class:`Styled`
     subclass."""
 
-    def __init__(self, id=None, style=None, parent=None):
+    def __init__(self, id=None, style=None, parent=None, source=None):
         """Associates `style` with this element. If `style` is `None`, an empty
         :class:`Style` is create, effectively using the defaults defined for the
         associated :class:`Style` class).
         A `parent` can be passed on object initialization, or later by
         assignment to the `parent` attribute."""
-        super().__init__(id=id, parent=parent)
+        super().__init__(id=id, parent=parent, source=source)
         if (isinstance(style, Style)
                 and not isinstance(style, (self.style_class, ParentStyle))):
             raise TypeError('the style passed to {} should be of type {} '
@@ -451,12 +460,11 @@ class Styled(DocumentElement, Configurable, metaclass=StyledMeta):
             return 0
 
     def configuration_name(self, document):
-        try:
-            # raise AttributeError
+        try:    # TODO: make document hashable so @cached can be used?
             return self._style_name
         except AttributeError:
             ruleset = self.configuration_class.get_ruleset(document)
-            self._style_name = ruleset.find_style(self)
+            self._style_name = ruleset.find_style(self, document)
             return self._style_name
 
     def fallback_to_parent(self, attribute):
@@ -473,10 +481,12 @@ class Styled(DocumentElement, Configurable, metaclass=StyledMeta):
 
     @property
     def has_class(self):
+        """Filter selection on a class of this :class:`Styled`"""
         return HasClass(self)
 
     @property
     def has_classes(self):
+        """Filter selection on a set of classes of this :class:`Styled`"""
         return HasClasses(self)
 
     def before_placing(self, container):
@@ -556,7 +566,7 @@ class StyledMatcher(dict):
         for name, selector in dict(iterable or (), **kwargs).items():
             self[name] = selector
 
-    def match(self, styled, stylesheet):
+    def match(self, styled, stylesheet, document):
         for cls in type(styled).__mro__:
             if cls not in self:
                 continue
@@ -564,7 +574,7 @@ class StyledMatcher(dict):
             for style in set((style_str, None)):
                 for name, selector in self[cls].get(style, {}).items():
                     selector = selector.flatten(stylesheet)
-                    specificity = selector.match(styled, stylesheet)
+                    specificity = selector.match(styled, stylesheet, document)
                     if specificity:
                         yield Match(name, specificity)
 
@@ -591,19 +601,19 @@ class StyleSheet(RuleSet, Resource):
     main_section = 'STYLESHEET'
     extension = '.rts'
 
-    def __init__(self, name, matcher=None, base=None, description=None,
-                 pygments_style=None, **user_options):
+    def __init__(self, name, matcher=None, base=None, source=None,
+                 description=None, pygments_style=None, **user_options):
         from .highlight import pygments_style_to_stylesheet
         from .stylesheets import matcher as default_matcher
 
-        base = self.from_string(base) if isinstance(base, str) else base
+        base = self.from_string(base, self) if isinstance(base, str) else base
         if matcher is None:
             matcher = default_matcher if base is None else StyledMatcher()
         if matcher is not None:
             matcher.check_validity()
         if pygments_style:
             base = pygments_style_to_stylesheet(pygments_style, base)
-        super().__init__(name, base=base)
+        super().__init__(name, base=base, source=source)
         self.description = description
         self.matcher = matcher
         if user_options:
@@ -618,20 +628,17 @@ class StyleSheet(RuleSet, Resource):
         raise NotImplementedError
 
     @classmethod
-    def parse_string(cls, filename_or_resource_name):
-        stylesheet_path = Path(filename_or_resource_name)
-        if stylesheet_path.is_file():
-            return StyleSheetFile(filename_or_resource_name)
-        else:
-            return super().parse_string(filename_or_resource_name)
+    def parse_string(cls, string, source):
+        with suppress(ResourceNotFound):
+            return super().parse_string(string, source)
+        return StyleSheetFile(string, source=source)
 
     @classmethod
     def doc_repr(cls, value):
-        for name, entry_point in cls.installed_resources:
-            if value is entry_point.load():
-                object_name, = entry_point.attrs
+        for name, ep in cls.installed_resources:
+            if value is ep.load():
                 return ('``{}`` (= :data:`{}.{}`)'
-                        .format(name, entry_point.module_name, object_name))
+                        .format(name, *ep.value.split(':')))
         raise NotImplementedError
 
     @classmethod
@@ -677,14 +684,14 @@ class StyleSheet(RuleSet, Resource):
             else:
                 raise KeyError("No selector found for style '{}'".format(name))
 
-    def find_matches(self, styled):
-        for match in self.matcher.match(styled, self):
+    def find_matches(self, styled, document):
+        for match in self.matcher.match(styled, self, document):
             yield match
         if self.base is not None:
-            yield from self.base.find_matches(styled)
+            yield from self.base.find_matches(styled, document)
 
-    def find_style(self, styled):
-        matches = sorted(self.find_matches(styled),
+    def find_style(self, styled, document):
+        matches = sorted(self.find_matches(styled, document),
                          key=attrgetter('specificity'), reverse=True)
         last_match = Match(None, ZERO_SPECIFICITY)
         for match in matches:
@@ -765,7 +772,13 @@ class StyleSheetFile(RuleSetFile, StyleSheet):
                 pass
             style_cls = styled_class.style_class
         else:
-            style_cls = self.get_entry_class(style_name)
+            try:
+                style_cls = self.get_entry_class(style_name)
+            except KeyError:
+                warn("The style definition '{}' will be ignored since there"
+                     " is no selector defined for it in the matcher."
+                     .format(style_name))
+                return
         self[style_name] = style_cls(**dict(items))
 
 
@@ -824,25 +837,34 @@ def parse_class_selector(chars):
 
 def parse_selector_args(chars):
     args, kwargs = [], {}
-    assert next(chars) == '('
+    if next(chars) != '(':
+        raise StyleParseError('Expecting an opening brace')
     eat_whitespace(chars)
     while chars.peek() not in (None, ')'):
-        argument = parse_value(chars)
-        if argument is not None:
-            assert not kwargs
-            args.append(argument)
-        else:
-            keyword = parse_keyword(chars)
+        argument, unknown_keyword = parse_value(chars)
+        eat_whitespace(chars)
+        if chars.peek() == '=':
+            next(chars)
+            keyword = argument
+            if not unknown_keyword:
+                raise StyleParseError("'{}' is not a valid keyword argument"
+                                      .format(keyword))
             eat_whitespace(chars)
-            kwargs[keyword] = parse_value(chars)
+            argument, unknown_keyword = parse_value(chars)
+            kwargs[keyword] = argument
+        elif kwargs:
+            raise StyleParseError('Non-keyword argument cannot follow a '
+                                  'keyword argument')
+        else:
+            args.append(argument)
+        if unknown_keyword:
+            raise StyleParseError("Unknown keyword '{}'".format(argument))
         eat_whitespace(chars)
-        char = next(chars)
-        if char == ')':
-            break
-        assert char == ','
-        eat_whitespace(chars)
-    else:
-        assert next(chars) == ')'
+        if chars.peek() == ',':
+            next(chars)
+            eat_whitespace(chars)
+    if chars.peek() is None or next(chars) != ')':
+        raise StyleParseError('Expecting a closing brace')
     return args, kwargs
 
 
@@ -882,24 +904,16 @@ class CharIterator(str):
             return None
 
 
-def parse_keyword(chars):
-    keyword = chars.match(string.ascii_letters + string.digits + '_')
-    eat_whitespace(chars)
-    if chars.peek() != '=':
-        raise StyleParseError('Expecting an equals sign to follow a keyword')
-    next(chars)
-    return keyword
-
-
 def parse_value(chars):
+    unknown_keyword = False
     first_char = chars.peek()
     if first_char in ("'", '"'):
         value = parse_string(chars)
     elif first_char.isnumeric() or first_char in '+-':
         value = parse_number(chars)
     else:
-        value = None
-    return value
+        value, unknown_keyword = parse_keyword(chars)
+    return value, unknown_keyword
 
 
 def parse_string(chars):
@@ -925,6 +939,17 @@ def parse_number(chars):
     return literal_eval(chars.match('0123456789.e+-'))
 
 
+def parse_keyword(chars):
+    keyword = chars.match(string.ascii_letters + string.digits + '_')
+    try:
+        return KEYWORDS[keyword.lower()], False
+    except KeyError:
+        return keyword, True
+
+
+KEYWORDS = dict(true=True, false=False, none=None)
+
+
 class Specificity(namedtuple('Specificity',
                              ['priority', 'location', 'style', 'attributes',
                               'klass'])):
@@ -972,7 +997,7 @@ class StyleLog(object):
         self.entries = []
 
     def log_styled(self, styled, container, continued, custom_message=None):
-        matches = self.stylesheet.find_matches(styled)
+        matches = self.stylesheet.find_matches(styled, container.document)
         log_entry = StyleLogEntry(styled, container, matches, continued,
                                   custom_message)
         self.entries.append(log_entry)
@@ -1001,11 +1026,8 @@ class StyleLog(object):
                 attrs = OrderedDict()
                 style = None
                 indent = '  ' * level
-                if styled.parent and (styled.source.location
-                                      != styled.parent.source.location):
-                    location = '   ' + styled.source.location
-                else:
-                    location = ''
+                location = ('   ' + styled.source.location if styled.source
+                            else '')
                 continued_text = '(continued) ' if entry.continued else ''
                 log.write('  {}{}{}{}'
                           .format(indent, continued_text,
