@@ -8,8 +8,8 @@ from .flowable import Flowable, Float
 from .flowable import GroupedFlowables, StaticGroupedFlowables
 from .flowable import LabeledFlowable, GroupedLabeledFlowables
 from .flowable import DummyFlowable, AnchorFlowable, WarnFlowable
-from .flowable import SetMetadataFlowable, AddToFrontMatter
-from .highlight import CodeBlock, Token
+from .flowable import SetMetadataFlowable, SetOutOfLineFlowables
+from .highlight import CodeBlock, CodeBlockWithCaption, Token
 from .index import IndexSection, Index, IndexLabel, IndexEntry
 from .index import InlineIndexTarget, IndexTarget
 from .inline import InlineFlowable
@@ -27,6 +27,7 @@ from .structure import Section, TableOfContentsSection, Heading
 from .structure import ListOfEntry
 from .structure import Admonition
 from .structure import TableOfContents, TableOfContentsEntry
+from .structure import OutOfLineFlowables
 from .table import TableWithCaption, Table, TableSection, TableHead, TableBody
 from .table import TableRow, TableCell, TableCellBackground, TableCellBorder
 from .table import ListOfTables, ListOfTablesSection
