@@ -35,6 +35,7 @@ matcher = StyledMatcher({
     'strong': StyledText.like('strong'),
     'literal emphasis': StyledText.like('literal emphasis'),
     'literal strong': StyledText.like('literal strong'),
+    'inline math': StyledText.like('math'),
 
     'quote': StyledText.like('quote'),
     'file path': StyledText.like('file path'),
@@ -66,11 +67,15 @@ matcher('internal hyperlink', StyledText.like('internal link'))
 matcher('external hyperlink', StyledText.like('external link'))
 matcher('broken hyperlink', StyledText.like('broken link'))
 
+matcher('glossary inline definition',
+        StyledText.like('glossary inline definition'))
 
 # paragraphs
 
 matcher('body', Paragraph)
 matcher('code block', +CodeBlock)
+matcher('code block caption', CodeBlockWithCaption / Caption)
+matcher('math block', Paragraph.like('math'))
 matcher('attribution', Paragraph.like('attribution'))
 matcher('centered', Paragraph.like('centered'))
 matcher('line block', Paragraph.like('line block'))
@@ -119,11 +124,14 @@ matcher('sidebar subtitle', 'sidebar' / Paragraph.like('subtitle'))
 # lists
 
 matcher('list item label', ListItemLabel)
+matcher('list item body', ListItem / GroupedFlowables)
+matcher('list item paragraph', 'list item body' / Paragraph)
 
 matcher('enumerated list', List.like('enumerated'))
 matcher('enumerated list item', 'enumerated list' / ListItem)
 matcher('enumerated list item label', 'enumerated list item' / ListItemLabel)
-matcher('nested enumerated list', ListItem / 'enumerated list')
+matcher('nested enumerated list', SelectorByName('list item body')
+                                  / 'enumerated list')
 
 matcher('(table) enumerated list', TableCell / ... / 'enumerated list')
 matcher('(table) enumerated list item', '(table) enumerated list' / ListItem)
@@ -134,7 +142,8 @@ matcher('bulleted list', List.like('bulleted'))
 matcher('compact bulleted list', List.like('bulleted', compact=True))
 matcher('bulleted list item', 'bulleted list' / ListItem)
 matcher('bulleted list item label', 'bulleted list item' / ListItemLabel)
-matcher('nested bulleted list', ListItem / 'bulleted list')
+matcher('nested bulleted list', SelectorByName('list item body')
+                                / 'bulleted list')
 
 matcher('(table) bulleted list', TableCell / ... / 'bulleted list')
 matcher('(table) bulleted list item', '(table) bulleted list' / ListItem)
@@ -157,9 +166,6 @@ matcher('choices list', List.like('choices'))
 matcher('choices list item', 'choices list' / ListItem)
 matcher('choices list item label', 'choices list item' / ListItemLabel)
 
-matcher('list item body', ListItem / GroupedFlowables)
-matcher('list item paragraph', 'list item body' / Paragraph)
-
 matcher('definition list', DefinitionList)
 matcher('definition list item', 'definition list' / LabeledFlowable)
 matcher('definition term', 'definition list item'
@@ -276,10 +282,11 @@ matcher('footnote label', Note / Paragraph)
 
 # images & figures
 
-matcher('figure', Figure)
 matcher('image', Image)
 matcher('inline image', InlineImage)
-matcher('caption', Caption)
+matcher('figure', Figure)
+matcher('figure image', 'figure' / Image)
+matcher('figure caption', 'figure' / Caption)
 matcher('figure legend', 'figure' / GroupedFlowables.like('legend'))
 matcher('figure legend paragraph', 'figure legend' / Paragraph)
 
@@ -290,6 +297,7 @@ matcher('table of contents section', Section.like('table of contents'))
 matcher('table of contents title', 'table of contents section'
                                    / Heading.like(level=1))
 matcher('table of contents', TableOfContents)
+matcher('local table of contents', TableOfContents.like(local=True))
 matcher('toc level 1', TableOfContentsEntry.like(depth=1))
 matcher('toc level 2', TableOfContentsEntry.like(depth=2))
 matcher('toc level 3', TableOfContentsEntry.like(depth=3))
@@ -311,6 +319,7 @@ matcher('list of tables entry', 'list of tables' / ListOfEntry)
 
 matcher('table', Table)
 matcher('table with caption', TableWithCaption)
+matcher('table caption', 'table with caption' / Caption)
 matcher('choices table', Table.like('choice'))
 matcher('table cell', Table / TableSection / TableRow / TableCell)
 matcher('table body cell background on even row',
