@@ -11,18 +11,18 @@ import re
 from collections import OrderedDict
 from functools import partial
 from itertools import chain
-from pathlib import Path
 
 from . import styleds, reference
 from .attribute import (Bool, Integer, Attribute, AttributesDictionary,
                         RuleSet, RuleSetFile, WithAttributes, AttributeType,
                         OptionSet, AcceptNoneAttributeType, OverrideDefault,
-                        Configurable, DefaultValueException)
+                        Configurable, DefaultValueException,
+                        VariableNotDefined)
 from .dimension import Dimension, CM, PT, PERCENT
 from .document import Document, Page, PageOrientation, PageType
 from .element import create_destination
 from .image import BackgroundImage, Image
-from .flowable import Flowable
+from .flowable import Flowable, StaticGroupedFlowables
 from .language import Language, EN
 from .layout import (Container, DownExpandingContainer, UpExpandingContainer,
                      FlowablesContainer, FootnoteContainer, ChainedContainer,
@@ -37,8 +37,7 @@ from .resource import Resource
 from .text import StyledText, Tab
 from .strings import StringCollection, Strings
 from .structure import Header, Footer, HorizontalRule, NewChapterException
-from .style import (StyleSheet, CharIterator, parse_string, Specificity,
-                    DocumentLocationType)
+from .style import StyleSheet, Specificity, DocumentLocationType
 from .stylesheets import sphinx
 from .util import NamedDescriptor
 
@@ -105,11 +104,12 @@ class FlowablesList(AcceptNoneAttributeType):
         return value is None or all(isinstance(val, Flowable) for val in value)
 
     @classmethod
-    def parse_string(cls, string):
+    def parse_string(cls, string, source):
         locals = {}
         locals.update(reference.__dict__)
         locals.update(styleds.__dict__)
-        return eval(string, {'__builtins__':{}}, locals)
+        flowables = eval(string, {'__builtins__':{}}, locals)    # TODO: parse!
+        return [StaticGroupedFlowables(flowables, source=source)]
 
     @classmethod
     def doc_format(cls):
@@ -312,7 +312,7 @@ class DocumentPartTemplate(Template):
 
     @property
     def doc_repr(self):
-        doc = ('**{}** (:class:`{}.{}`)\n\n'\
+        doc = ('**{}** (:class:`{}.{}`)\n\n'
                .format(self.name, type(self).__module__, type(self).__name__))
         for name, default_value in self.items():
             doc += '  - *{}*: ``{}``\n'.format(name, default_value)
@@ -479,37 +479,26 @@ class TemplateConfiguration(RuleSet):
     template = None
     """The :class:`.DocumentTemplate` subclass to configure"""
 
-    def __init__(self, name, base=None, template=None, description=None,
-                 **options):
+    def __init__(self, name, base=None, source=None,
+                 template=None, description=None, **options):
         if template:
             if isinstance(template, str):
                 template = DocumentTemplate.from_string(template)
             assert self.template in (None, template)
             self.template = template
-        try:
-            stylesheet_path = Path(options.get('stylesheet'))
-            if not stylesheet_path.is_absolute():
-                stylesheet_path = self._stylesheet_search_path / stylesheet_path
-            if stylesheet_path.exists():
-                options['stylesheet'] = str(stylesheet_path)
-        except TypeError:
-            pass
         tmpl_cls = self.template
-        for attr, value in options.items():
-            options[attr] = tmpl_cls.validate_attribute(attr, value, True)
-        super().__init__(name, base=base or self.template, **options)
+        for attr, val in options.items():
+            options[attr] = self._validate_attribute(tmpl_cls, attr, val)
+        base = base or self.template
+        super().__init__(name, base=base, source=source, **options)
         self.description = description
-        self.variables['paper_size'] = A4
-
-    @property
-    def _stylesheet_search_path(self):
-        return Path.cwd()
 
     def get_entries(self, name, document):
         if name in self:
             yield self[name]
         if self.base:
             yield from self.base.get_entries(name, document)
+        raise KeyError(name)
 
     def get_attribute_value(self, name):
         if name in self:
@@ -541,15 +530,11 @@ class TemplateConfigurationFile(RuleSetFile, TemplateConfiguration):
 
     main_section = 'TEMPLATE_CONFIGURATION'
 
-    @property
-    def _stylesheet_search_path(self):
-        return self.filename.parent
-
     def process_section(self, section_name, classifier, items):
         if section_name in StringCollection.subclasses:
             collection_cls = StringCollection.subclasses[section_name]
             strings = self.setdefault('strings', Strings())
-            collection_items = {name: parse_string(CharIterator(value))
+            collection_items = {name: StyledText.from_string(value)
                                 for name, value in items}
             strings[collection_cls] = collection_cls(**collection_items)
         else:
@@ -653,14 +638,13 @@ class PartsList(AttributeType, list):
         return all(isinstance(item, str) for item in value)
 
     @classmethod
-    def parse_string(cls, string):
+    def parse_string(cls, string, source):
         return cls(*string.split())
 
     @classmethod
     def doc_repr(cls, value):
-        space = '\\ ``\N{NO-BREAK SPACE}``\\ '
-        return space.join(':attr:`{}`'.format(part)
-                          for part in value) or '(empty list)'
+        return ' '.join(':attr:`{}`'.format(part)
+                        for part in value) or '(empty list)'
 
     @classmethod
     def doc_format(cls):
@@ -689,6 +673,8 @@ class DocumentTemplate(Document, AttributesDictionary, Resource,
 
     parts = Attribute(PartsList, [], 'The parts making up this document')
 
+    variables = {'paper_size': A4}      # default variable values
+
     def __init__(self, document_tree, configuration=None, backend=None):
         self.configuration = (configuration if configuration is not None
                               else self.Configuration('empty'))
@@ -699,8 +685,12 @@ class DocumentTemplate(Document, AttributesDictionary, Resource,
         super().__init__(document_tree, stylesheet, language, strings=strings,
                          backend=backend)
         parts = self.get_option('parts')
-        self.part_templates = [next(self._find_templates(name))
-                               for name in parts]
+        try:
+            self.part_templates = [next(self._find_templates(name))
+                                   for name in parts]
+        except KeyError as exc:
+            raise ValueError("The '{}' document template has no part named"
+                             " '{}'".format(type(self).__name__, *exc.args))
         self._to_insert = {}
 
     def _find_templates(self, name):
@@ -715,6 +705,15 @@ class DocumentTemplate(Document, AttributesDictionary, Resource,
 
     RE_PAGE = re.compile('^(.*)_(right|left)_page$')
 
+    @classmethod
+    def get_variable(cls, configuration_class, attribute, variable):
+        try:
+            value = cls.variables[variable.name]
+        except KeyError:
+            raise VariableNotDefined("Document template provides no default "
+                                     "for variable '{}'".format(variable.name))
+        return configuration_class.attribute_type(attribute).validate(value)
+
     def get_option(self, option_name):
         return self.configuration.get_attribute_value(option_name)
 
