@@ -31,8 +31,8 @@ from ast import literal_eval
 from html.entities import name2codepoint
 from token import NAME, STRING, NEWLINE, LPAR, RPAR, ENDMARKER
 
-from .attribute import (AttributeType, Attribute, Bool, Integer,
-                        AcceptNoneAttributeType, ParseError)
+from .attribute import (AttributeType, AcceptNoneAttributeType, Attribute,
+                        Bool, Integer)
 from .color import Color, BLACK
 from .dimension import Dimension, PT
 from .font import Typeface
@@ -43,13 +43,16 @@ from .style import Style, Styled, StyledMeta
 from .util import NotImplementedAttribute
 
 
-__all__ = ['TextStyle', 'StyledText', 'WarnInline', 'SingleStyledText',
-           'MixedStyledText', 'ConditionalMixedStyledText', 'Space',
-           'FixedWidthSpace', 'NoBreakSpace', 'Spacer', 'Tab', 'Newline',
-           'Superscript', 'Subscript']
+__all__ = ['InlineStyle', 'InlineStyled', 'TextStyle', 'StyledText',
+           'WarnInline', 'SingleStyledText', 'MixedStyledText',
+           'ConditionalMixedStyledText',
+           'Space', 'FixedWidthSpace', 'NoBreakSpace', 'Spacer', 'Tab',
+           'Newline', 'Superscript', 'Subscript']
 
 
 class Locale(AttributeType):
+    """Selects a language"""
+
     REGEX = re.compile(r'^[a-z]{2}_[A-Z]{2}$')
 
     @classmethod
@@ -57,7 +60,7 @@ class Locale(AttributeType):
         return cls.REGEX.match(value) is not None
 
     @classmethod
-    def from_tokens(cls, tokens):
+    def from_tokens(cls, tokens, source):
         token = next(tokens)
         if not cls.check_type(token.string):
             raise ValueError("'{}' is not a valid locale. Needs to be of the "
@@ -69,26 +72,25 @@ class Locale(AttributeType):
         return 'locale identifier in the ``<language ID>_<region ID>`` format'
 
 
-class TextStyle(Style):
-    typeface = Attribute(Typeface, adobe14.times, 'Typeface to set the text in')
-    font_weight = Attribute(FontWeight, 'medium', 'Thickness of character '
-                                                  'outlines relative to their '
-                                                  'height')
-    font_slant = Attribute(FontSlant, 'upright', 'Slope style of the font')
-    font_width = Attribute(FontWidth, 'normal', 'Stretch of the characters')
-    font_size = Attribute(Dimension, 10*PT, 'Height of characters')
-    font_color = Attribute(Color, BLACK, 'Color of the font')
-    font_variant = Attribute(FontVariant, 'normal', 'Variant of the font')
-    position = Attribute(TextPosition, 'normal', 'Vertical text position')
-    kerning = Attribute(Bool, True, 'Improve inter-letter spacing')
-    ligatures = Attribute(Bool, True, 'Run letters together where possible')
-    # TODO: character spacing
-    hyphenate = Attribute(Bool, True, 'Allow words to be broken over two lines')
-    hyphen_chars = Attribute(Integer, 2, 'Minimum number of characters in a '
-                                         'hyphenated part of a word')
-    hyphen_lang = Attribute(Locale, 'en_US', 'Language to use for hyphenation. '
-                                             'Accepts locale codes such as '
-                                             "'en_US'")
+class InlineStyled(Styled):
+    """"""
+
+    def wrapped_spans(self, container):
+        """Generator yielding all spans in this inline element (flattened)"""
+        before = self.get_style('before', container)
+        if before is not None:
+            yield from before.copy(self.parent).wrapped_spans(container)
+        if not self.get_style('hide', container):
+            yield from self.spans(container)
+        after = self.get_style('after', container)
+        if after is not None:
+            yield from after.copy(self.parent).wrapped_spans(container)
+
+    def copy(self, parent=None):
+        raise NotImplementedError
+
+    def spans(self, container):
+        raise NotImplementedError
 
 
 class CharacterLike(Styled):
@@ -110,11 +112,9 @@ NAME2CHAR = {name: chr(codepoint)
              for name, codepoint in name2codepoint.items()}
 
 
-class StyledText(Styled, AcceptNoneAttributeType):
+class StyledText(InlineStyled, AcceptNoneAttributeType):
     """Base class for text that has a :class:`TextStyle` associated with it."""
 
-    style_class = TextStyle
-
     def __add__(self, other):
         """Return the concatenation of this styled text and `other`. If `other`
         is `None`, this styled text itself is returned."""
@@ -135,46 +135,40 @@ class StyledText(Styled, AcceptNoneAttributeType):
         return super().check_type(value) or isinstance(value, str)
 
     @classmethod
-    def from_tokens(cls, tokens):
-        texts = []
-        for token in tokens:
-            if token.type == NEWLINE:
-                continue
-            elif token.type == NAME:      # inline flowable
-                directive = token.string.lower()
-                inline_flowable_class = InlineFlowable.directives[directive]
-                if next(tokens).exact_type != LPAR:
-                    raise ParseError('Expecting an opening parenthesis.')
-                args, kwargs = inline_flowable_class.arguments.parse_arguments(tokens)
-                if next(tokens).exact_type != RPAR:
-                    raise ParseError('Expecting a closing parenthesis.')
-                inline_flowable = inline_flowable_class(*args, **kwargs)
-                texts.append(inline_flowable)
-            elif token.type == STRING:    # text
-                text = literal_eval(token.string)
-                if tokens.next.exact_type == LPAR:
-                    _, start_col = next(tokens).end
-                    for token in tokens:
-                        if token.exact_type == RPAR:
-                            _, end_col = token.start
-                            style = token.line[start_col:end_col].strip()
-                            break
-                        elif token.type == NEWLINE:
-                            raise StyledTextParseError('No newline allowed in'
-                                                       'style name')
-                else:
-                    style = None
-                texts.append(cls._substitute_variables(text, style))
+    def from_tokens(cls, tokens, source):
+        items = []
+        while tokens.next.type:
+            if tokens.next.type == NAME:
+                items.append(InlineFlowable.from_tokens(tokens, source))
+            elif tokens.next.type == STRING:
+                items.append(cls.text_from_tokens(tokens))
+            elif tokens.next.type == NEWLINE:
+                next(tokens)
+            elif tokens.next.type == ENDMARKER:
+                break
             else:
                 raise StyledTextParseError('Expecting text or inline flowable')
-            if tokens.next.type == ENDMARKER:
-                break
-
-        if len(texts) > 1:
-            return MixedStyledText(texts)
-        else:
-            first, = texts
+        if len(items) == 1:
+            first, = items
             return first
+        return MixedStyledText(items, source=source)
+
+    @classmethod
+    def text_from_tokens(cls, tokens):
+        text = literal_eval(next(tokens).string)
+        if tokens.next.exact_type == LPAR:
+            _, start_col = next(tokens).end
+            for token in tokens:
+                if token.exact_type == RPAR:
+                    _, end_col = token.start
+                    style = token.line[start_col:end_col].strip()
+                    break
+                elif token.type == NEWLINE:
+                    raise StyledTextParseError('No newline allowed in '
+                                               'style name')
+        else:
+            style = None
+        return cls._substitute_variables(text, style)
 
     @classmethod
     def doc_repr(cls, value):
@@ -188,10 +182,10 @@ class StyledText(Styled, AcceptNoneAttributeType):
                 "``'text string' (style name)``")
 
     @classmethod
-    def validate(cls, value, accept_variables=False, attribute_name=None):
-        if attribute_name is None and isinstance(value, str):
+    def validate(cls, value):
+        if isinstance(value, str):
             value = SingleStyledText(value)
-        return super().validate(value, accept_variables, attribute_name)
+        return super().validate(value)
 
     @classmethod
     def _substitute_variables(cls, text, style):
@@ -220,7 +214,7 @@ class StyledText(Styled, AcceptNoneAttributeType):
         return self.parent.paragraph
 
     def fallback_to_parent(self, attribute):
-        return attribute != 'position'
+        return attribute not in ('position', 'before', 'after')
 
     position = {TextPosition.SUPERSCRIPT: 1 / 3,
                 TextPosition.SUBSCRIPT: - 1 / 6}
@@ -259,18 +253,47 @@ class StyledText(Styled, AcceptNoneAttributeType):
     @property
     def items(self):
         """The list of items in this StyledText."""
-        raise NotImplementedError
-
-    def spans(self, container):
-        """Generator yielding all spans in this styled text, one
-        item at a time (used in typesetting)."""
-        raise NotImplementedError
+        return [self]
 
 
 class StyledTextParseError(Exception):
     pass
 
 
+class InlineStyle(Style):
+    before = Attribute(StyledText, None, 'Item to insert before this one')
+    after = Attribute(StyledText, None, 'Item to insert after this one')
+
+
+InlineStyled.style_class = InlineStyle
+
+
+class TextStyle(InlineStyle):
+    typeface = Attribute(Typeface, adobe14.times, 'Typeface to set the text in')
+    font_weight = Attribute(FontWeight, 'medium', 'Thickness of character '
+                                                  'outlines relative to their '
+                                                  'height')
+    font_slant = Attribute(FontSlant, 'upright', 'Slope style of the font')
+    font_width = Attribute(FontWidth, 'normal', 'Stretch of the characters')
+    font_size = Attribute(Dimension, 10*PT, 'Height of characters')
+    font_color = Attribute(Color, BLACK, 'Color of the font')
+    font_variant = Attribute(FontVariant, 'normal', 'Variant of the font')
+    # TODO: text_case = Attribute(TextCase, None, 'Change text casing')
+    position = Attribute(TextPosition, 'normal', 'Vertical text position')
+    kerning = Attribute(Bool, True, 'Improve inter-letter spacing')
+    ligatures = Attribute(Bool, True, 'Run letters together where possible')
+    # TODO: character spacing
+    hyphenate = Attribute(Bool, True, 'Allow words to be broken over two lines')
+    hyphen_chars = Attribute(Integer, 2, 'Minimum number of characters in a '
+                                         'hyphenated part of a word')
+    hyphen_lang = Attribute(Locale, 'en_US', 'Language to use for hyphenation. '
+                                             'Accepts locale codes such as '
+                                             "'en_US'")
+
+
+StyledText.style_class = TextStyle
+
+
 class WarnInline(StyledText):
     """Inline element that emits a warning
 
@@ -286,10 +309,6 @@ class WarnInline(StyledText):
     def to_string(self, flowable_target):
         return ''
 
-    @property
-    def items(self):
-        return [self]
-
     def spans(self, container):
         self.warn(self.message, container)
         return iter([])
@@ -335,10 +354,6 @@ class SingleStyledTextBase(StyledText):
         return (self.font(container).line_gap_in_pt
                 * float(self.get_style('font_size', container)))
 
-    @property
-    def items(self):
-        return [self]
-
     def spans(self, container):
         yield self
 
@@ -349,8 +364,10 @@ ESCAPE = str.maketrans({"'": r"\'",
 
 
 class SingleStyledText(SingleStyledTextBase):
-    def __init__(self, text, style=None, parent=None):
-        super().__init__(style=style, parent=parent)
+    """Text with a single style applied"""
+
+    def __init__(self, text, style=None, parent=None, source=None):
+        super().__init__(style=style, parent=parent, source=source)
         self._text = text
 
     def __str__(self):
@@ -362,6 +379,10 @@ class SingleStyledText(SingleStyledTextBase):
     def text(self, container, **kwargs):
         return self._text
 
+    def copy(self, parent=None):
+        return type(self)(self._text, style=self.style, parent=parent,
+                          source=self.source)
+
 
 class MixedStyledTextBase(StyledText):
     def to_string(self, flowable_target):
@@ -373,8 +394,7 @@ class MixedStyledTextBase(StyledText):
         mixed-styled text."""
         for child in self.children(container):
             container.register_styled(child)
-            for span in child.spans(container):
-                yield span
+            yield from child.wrapped_spans(container)
 
     def children(self, flowable_target):
         raise NotImplementedError
@@ -392,7 +412,8 @@ class MixedStyledText(MixedStyledTextBase, list):
 
     _assumed_equal = {}
 
-    def __init__(self, text_or_items, id=None, style=None, parent=None):
+    def __init__(self, text_or_items, id=None, style=None, parent=None,
+                 source=None):
         """Initialize this mixed-styled text as the concatenation of
         `text_or_items`, which is either a single text item or an iterable of
         text items. Individual text items can be :class:`StyledText` or
@@ -400,7 +421,7 @@ class MixedStyledText(MixedStyledTextBase, list):
         each of the text items.
 
         See :class:`StyledText` for information on `style`, and `parent`."""
-        super().__init__(id=id, style=style, parent=parent)
+        super().__init__(id=id, style=style, parent=parent, source=source)
         if isinstance(text_or_items, (str, StyledText)):
             text_or_items = (text_or_items, )
         for item in text_or_items:
@@ -438,6 +459,7 @@ class MixedStyledText(MixedStyledTextBase, list):
             assumed_equal.remove(other_id)
 
     def prepare(self, flowable_target):
+        super().prepare(flowable_target)
         for item in self:
             item.prepare(flowable_target)
 
@@ -458,6 +480,11 @@ class MixedStyledText(MixedStyledTextBase, list):
     def children(self, flowable_target):
         return self.items
 
+    def copy(self, parent=None):
+        items = [item.copy() for item in self.items]
+        return type(self)(items, style=self.style, parent=parent,
+                          source=self.source)
+
 
 class ConditionalMixedStyledText(MixedStyledText):
     def __init__(self, text_or_items, document_option, style=None, parent=None):
@@ -559,14 +586,18 @@ class ControlCharacter(Character, metaclass=ControlCharacterMeta):
     exception = Exception
     all = {}
 
-    def __init__(self):
+    def __init__(self, style=None, parent=None, source=None):
         """Initialize this control character with it's unicode `char`."""
-        super().__init__(self.character)
+        super().__init__(self.character, style=style, parent=parent,
+                         source=source)
 
     def __repr__(self):
         """A textual representation of this control character."""
         return self.__class__.__name__
 
+    def copy(self, parent=None):
+        return type(self)(style=self.style, parent=parent, source=self.source)
+
     @property
     def width(self):
         """Raises the exception associated with this control character.
@@ -604,19 +635,21 @@ SUBSCRIPT_STYLE = TextStyle(position='subscript')
 class Superscript(MixedStyledText):
     """Superscript."""
 
-    def __init__(self, text, parent=None):
+    def __init__(self, text, parent=None, source=None):
         """Accepts a single instance of :class:`str` or :class:`StyledText`, or
         an iterable of these."""
-        super().__init__(text, style=SUPERSCRIPT_STYLE, parent=parent)
+        super().__init__(text, style=SUPERSCRIPT_STYLE, parent=parent,
+                         source=source)
 
 
 class Subscript(MixedStyledText):
     """Subscript."""
 
-    def __init__(self, text, parent=None):
+    def __init__(self, text, parent=None, source=None):
         """Accepts a single instance of :class:`str` or :class:`StyledText`, or
         an iterable of these."""
-        super().__init__(text, style=SUBSCRIPT_STYLE, parent=parent)
+        super().__init__(text, style=SUBSCRIPT_STYLE, parent=parent,
+                         source=source)
 
 
 from .reference import Field
