@@ -28,10 +28,12 @@ import time
 from collections import OrderedDict
 from collections.abc import MutableMapping
 from functools import wraps
+from itertools import tee
 from weakref import ref
 
 
-__all__ = ['all_subclasses', 'intersperse', 'PeekIterator', 'posix_path',
+__all__ = ['INF', 'all_subclasses', 'clamp', 'intersperse', 'itemcount',
+           'PeekIterator', 'posix_path',
            'consumer', 'cached', 'cached_property', 'cached_generator',
            'class_property', 'timed', 'Decorator', 'ReadAliasAttribute',
            'NotImplementedAttribute', 'NamedDescriptor',
@@ -39,6 +41,11 @@ __all__ = ['all_subclasses', 'intersperse', 'PeekIterator', 'posix_path',
            'VersionError']
 
 
+# constants
+
+INF = float('+inf')
+
+
 # functions
 
 def all_subclasses(cls):
@@ -49,6 +56,11 @@ def all_subclasses(cls):
             yield subsubcls
 
 
+def clamp(min_value, value, max_value):
+    """Limit `value` to a range between `min_value` and `max_value`"""
+    return max(min_value, min(value, max_value))
+
+
 def intersperse(iterable, element):
     """Generator yielding all elements of `iterable`, but with `element`
     inserted between each two consecutive elements"""
@@ -59,6 +71,11 @@ def intersperse(iterable, element):
         yield next_from_iterable
 
 
+def itemcount(iterable):
+    """Count the number of items in `iterable`"""
+    return sum(1 for _ in iterable)
+
+
 class PeekIterator(object):
     """An iterator that allows inspecting the next element"""
 
@@ -67,6 +84,24 @@ class PeekIterator(object):
         self._iterator = iter(iterable)
         self.at_end = False
         self._advance()
+        self._saved_states = []
+
+    def push_state(self):
+        """Save the current state on a stack"""
+        self._iterator, saved_iter = tee(self._iterator)
+        state = saved_iter, self.next, self.at_end
+        self._saved_states.append(state)
+
+    def pop_state(self, discard=False):
+        """Restore or discard the last saved state
+
+        Args:
+            discard (bool): if True, discard the last saved state
+
+        """
+        state = self._saved_states.pop()
+        if not discard:
+            self._iterator, self.next, self.at_end = state
 
     def _advance(self):
         result = self.next
