@@ -14,6 +14,7 @@ from rinoh.dimension import (Dimension, PT, PICA, INCH, MM, CM,
                              PERCENT, QUARTERS)
 from rinoh.draw import Stroke
 from rinoh.flowable import FlowableWidth, HorizontalAlignment, Break
+from rinoh.font.style import ClassSet, FontWeight, FontSlant, FontWidth
 from rinoh.image import BackgroundImage, Scale, InlineImage
 from rinoh.number import NumberFormat
 from rinoh.paper import Paper, A4, A5, JUNIOR_LEGAL
@@ -22,12 +23,12 @@ from rinoh.paragraph import (Paragraph, TextAlign, TabAlign,
                              ProportionalSpacing, FixedSpacing, Leading)
 from rinoh.reference import (ReferenceField, ReferenceText, Field, PAGE_NUMBER,
                              NUMBER_OF_PAGES, SECTION_TITLE, SECTION_NUMBER)
-from rinoh.strings import StringField
+from rinoh.strings import StringField, UserStrings
 from rinoh.structure import SectionTitles, AdmonitionTitles
 from rinoh.style import (SelectorByName, parse_selector, parse_selector_args,
                          parse_class_selector, parse_keyword, parse_string,
                          parse_number, StyleParseError, CharIterator)
-from rinoh.table import VerticalAlign
+from rinoh.table import ColumnWidths, VerticalAlign
 from rinoh.text import StyledText, SingleStyledText, MixedStyledText, Tab
 
 
@@ -35,13 +36,17 @@ def test_optionset_from_string():
     ONE = 'one'
     TWO = 'two'
     THREE = 'three'
+    WITH_SPACE = 'with space'
+    DOUBLE_SPACE = 'double  space'
 
     class TestSet1(OptionSet):
-        values = ONE, TWO, THREE
+        values = ONE, TWO, THREE, WITH_SPACE, DOUBLE_SPACE
 
     assert TestSet1.from_string('one') == ONE
     assert TestSet1.from_string('TWO') == TWO
     assert TestSet1.from_string('tHRee') == THREE
+    assert TestSet1.from_string('With Space') == WITH_SPACE
+    assert TestSet1.from_string('double  SPACE') == DOUBLE_SPACE
     with pytest.raises(ValueError):
         TestSet1.from_string('four')
     with pytest.raises(ValueError):
@@ -60,6 +65,62 @@ def test_optionset_from_string():
         TestSet2.from_string('False')
 
 
+def test_classset_from_string():
+    class TestClassSet(ClassSet):
+        classes = {
+            1: ['one', 'uno'],
+            3: ['number three', 'tres'],
+            4: ['four', 'numero-quattro']
+        }
+
+    assert TestClassSet.from_string('one') == 1
+    assert TestClassSet.from_string('uNo') == 1
+    assert TestClassSet.from_string('number Three') == 3
+    assert TestClassSet.from_string('tres') == 3
+    assert TestClassSet.from_string('fOUr') == 4
+    assert TestClassSet.from_string('numero-quattro') == 4
+    assert TestClassSet.from_string('numero quattro') == 4
+    assert TestClassSet.from_string('Numero Quattro') == 4
+    assert TestClassSet.from_string('numeroquattro') == 4
+    assert TestClassSet.from_string('numeroQuattro') == 4
+    with pytest.raises(ValueError):
+        TestClassSet.from_string('number-three')
+
+
+def test_fontweight_from_string():
+    assert FontWeight.from_string('regular') == FontWeight.REGULAR
+    assert FontWeight.from_string('normal') == FontWeight.REGULAR
+    assert FontWeight.from_string('semibold') == FontWeight.SEMI_BOLD
+    assert FontWeight.from_string('semi-bold') == FontWeight.SEMI_BOLD
+    assert FontWeight.from_string('semi bold') == FontWeight.SEMI_BOLD
+    assert FontWeight.from_string('400') == FontWeight.REGULAR
+    assert FontWeight.from_string('123') == 123
+    with pytest.raises(ValueError):
+        FontWeight.from_string('1299')
+
+
+def test_fontslant_from_string():
+    assert FontSlant.from_string('upright') == FontSlant.UPRIGHT
+    assert FontSlant.from_string('obLIQUE') == FontSlant.OBLIQUE
+    assert FontSlant.from_string('Italic') == FontSlant.ITALIC
+    with pytest.raises(ValueError):
+        FontSlant.from_string('somewhat-oblique')
+
+
+def test_fontwidth_from_string():
+    assert FontWidth.from_string('normal') == FontWidth.NORMAL
+    assert FontWidth.from_string('Medium') == FontWidth.MEDIUM
+    assert FontWidth.from_string('condenSED') == FontWidth.CONDENSED
+    assert FontWidth.from_string('semi-expanded') == FontWidth.SEMI_EXPANDED
+    assert FontWidth.from_string('ultra eXpanded') == FontWidth.ULTRA_EXPANDED
+    assert FontWidth.from_string('UltraCondensed') == FontWidth.ULTRA_CONDENSED
+    assert FontWidth.from_string('5') == FontWidth.NORMAL
+    with pytest.raises(ValueError):
+        FontWidth.from_string('0')
+    with pytest.raises(ValueError):
+        FontWidth.from_string('11')
+
+
 def test_numberformat_from_string():
     assert NumberFormat.from_string('none') == None
     assert NumberFormat.from_string('number') == NumberFormat.NUMBER
@@ -201,7 +262,7 @@ def test_dimension_from_string():
     assert Dimension.from_string('21%') == 21*PERCENT
     assert Dimension.from_string('-16.12%') == -16.12*PERCENT
     assert Dimension.from_string('3/4') == 3*QUARTERS
-    with pytest.raises(ValueError):
+    with pytest.raises(ParseError):
         assert Dimension.from_string('20inch')
 
 
@@ -304,6 +365,10 @@ def test_styledtext_from_string_field():
            == MixedStyledText([SingleStyledText('1'),
                                StringField(AdmonitionTitles, 'warning'),
                                SingleStyledText('2')], style='style')
+    assert StyledText.from_string("'{UserStrings.my_string}abc' (style)") \
+           == MixedStyledText([StringField(UserStrings, 'my_string'),
+                               SingleStyledText('abc')], style='style')
+
 
 def test_styledtext_from_string_inline_image():
     assert StyledText.from_string("IMAGE('images/image.pdf')") \
@@ -383,22 +448,43 @@ def test_backgroundimage_from_string():
         BackgroundImage.from_string('not_a_path')
 
 
+def test_column_widths_from_string():
+    assert ColumnWidths.from_string('none') is None
+    assert ColumnWidths.from_string('auto') == [None]
+    assert ColumnWidths.from_string('1 2 3') == [1, 2, 3]
+    assert ColumnWidths.from_string('1 0 9') == [1, 0, 9]
+    assert ColumnWidths.from_string('2      4  6') == [2, 4, 6]
+    assert ColumnWidths.from_string('6  5   4   ') == [6, 5, 4]
+    assert ColumnWidths.from_string('1pt 2cm 3in') == [1*PT, 2*CM, 3*INCH]
+    assert ColumnWidths.from_string('4 pt 5 cm 6 in') == [4*PT, 5*CM, 6*INCH]
+    assert ColumnWidths.from_string('7pt 8 cm 9in') == [7*PT, 8*CM, 9*INCH]
+    assert ColumnWidths.from_string('10 20 1cm 30') == [10, 20, 1*CM, 30]
+    assert ColumnWidths.from_string('1 2cm auto') == [1, 2*CM, None]
+    assert ColumnWidths.from_string('2 auto 5') == [2, None, 5]
+    with pytest.raises(ParseError):
+        assert ColumnWidths.from_string('autoo')
+    with pytest.raises(ParseError):
+        assert ColumnWidths.from_string('1 none')
+    with pytest.raises(ParseError):
+        assert ColumnWidths.from_string('1.5 3 1cm')
+
+
 # selectors
 
 def test_parse_keyword():
     def helper(string):
         chars = CharIterator(string)
-        return parse_keyword(chars), ''.join(chars)
+        keyword, unknown_keyword = parse_keyword(chars)
+        return keyword, unknown_keyword, ''.join(chars)
 
-    assert helper('style=') == ('style', '')
-    assert helper('row =') == ('row', '')
-    assert helper('UPPERCASE=') == ('UPPERCASE', '')
-    assert helper('miXeDCaSE=') == ('miXeDCaSE', '')
-    assert helper('key =  efzef') == ('key', '  efzef')
-    with pytest.raises(StyleParseError):
-        helper('bad key =')
-    with pytest.raises(StyleParseError):
-        helper('bad')
+    assert helper('None') == (None, False, '')
+    assert helper('trUE =') == (True, False, ' =')
+    assert helper('FALSE doh!') == (False, False, ' doh!')
+    assert helper('style') == ('style', True, '')
+    assert helper('row ') == ('row', True, ' ')
+    assert helper('UPPERCASE') == ('UPPERCASE', True, '')
+    assert helper('miXeDCaSE') == ('miXeDCaSE', True, '')
+    assert helper('key   efzef') == ('key', True, '   efzef')
 
 
 def test_parse_string():
@@ -444,6 +530,9 @@ def test_parse_selector_args():
     assert helper("666") == ([666], {})
     assert helper("'baboo', ") == (['baboo'], {})
     assert helper("22, ") == ([22], {})
+    assert helper("true") == ([True], {})
+    assert helper("False,") == ([False], {})
+    assert helper("nOnE") == ([None], {})
     assert helper("'style name', 666") == (['style name', 666], {})
     assert helper("'style name' ,666") == (['style name', 666], {})
     assert helper("'style name',666") == (['style name', 666], {})
@@ -455,6 +544,12 @@ def test_parse_selector_args():
     assert helper("key_9='value'") == ([], dict(key_9='value'))
     assert helper("'arg', key='value'") == (['arg'], dict(key='value'))
     assert helper("22, key='value'") == ([22], dict(key='value'))
+    assert helper("key =true") == ([], dict(key=True))
+    assert helper("42, key=falSe") == ([42], dict(key=False))
+    with pytest.raises(StyleParseError):
+        assert helper("key=bad")
+    with pytest.raises(StyleParseError):
+        assert helper("key='good', 33")
 
 
 def test_parse_class_selector():
